
import os
import shutil
import logging

from datetime import datetime, timedelta
from .newsml_1_2 import Parser
from ..utc import utc, utcnow
from ..etree import etree
from superdesk.notification import push_notification
from superdesk.io import register_provider, IngestService

logger = logging.getLogger(__name__)
PROVIDER = 'afp'


def is_ready(last_updated, provider_last_updated=None):
    """Parse file only if it's not older than provider last update -10m"""

    if not provider_last_updated:
        provider_last_updated = utcnow() - timedelta(days=7)

    return provider_last_updated - timedelta(minutes=10) < last_updated


class AFPIngestService(IngestService):
    """AFP Ingest Service"""

    def __init__(self):
        self.parser = Parser()

    def update(self, provider):
        self.provider = provider
        self.path = provider.get('config', {}).get('path', None)
        if not self.path:
            return

        try:
            for filename in os.listdir(self.path):
                if os.path.isfile(os.path.join(self.path, filename)):
                    filepath = os.path.join(self.path, filename)
                    stat = os.lstat(filepath)
                    last_updated = datetime.fromtimestamp(stat.st_mtime, tz=utc)
                    if is_ready(last_updated, provider.get('updated')):
                        with open(os.path.join(self.path, filename), 'r') as f:
                            item = Parser().parse_message(etree.fromstring(f.read()))
                            item['_created'] = item['firstcreated'] = utc.localize(item['firstcreated'])
                            item['_updated'] = item['versioncreated'] = utc.localize(item['versioncreated'])
                            item.setdefault('provider', provider.get('name', provider['type']))
                            self.move_the_current_file(filename, success=True)
                            yield [item]
        except (Exception) as err:
            logger.exception(err)
            self.move_the_current_file(filename, success=False)
            pass
        finally:
            push_notification('ingest:update')

    def move_the_current_file(self, filename, success=True):
        try:
            if not os.path.exists(os.path.join(self.path, "_PROCESSED/")):
                os.makedirs(os.path.join(self.path, "_PROCESSED/"))
            if not os.path.exists(os.path.join(self.path, "_ERROR/")):
                os.makedirs(os.path.join(self.path, "_ERROR/"))

            if success:
                shutil.copy2(os.path.join(self.path, filename), os.path.join(self.path, "_PROCESSED/"))
            else:
                shutil.copy2(os.path.join(self.path, filename), os.path.join(self.path, "_ERROR/"))
        finally:
            os.remove(os.path.join(self.path, filename))

register_provider(PROVIDER, AFPIngestService())




import logging
from flask import current_app as app


log = logging.getLogger(__name__)


class EveBackend():
    def find_one(self, endpoint_name, req, **lookup):
        backend = self._lookup_backend(endpoint_name, fallback=True)
        return backend.find_one(endpoint_name, req=req, **lookup)

    def find_one_in_base_backend(self, endpoint_name, req, **lookup):
        backend = self._backend(endpoint_name)
        return backend.find_one(endpoint_name, req=req, **lookup)

    def get(self, endpoint_name, req, lookup):
        backend = self._lookup_backend(endpoint_name, fallback=True)
        cursor = backend.find(endpoint_name, req, lookup)
        if not cursor.count():
            return cursor  # return 304 if not modified
        else:
            # but fetch without filter if there is a change
            req.if_modified_since = None
            return backend.find(endpoint_name, req, lookup)

    def create(self, endpoint_name, docs, **kwargs):
        backend = self._backend(endpoint_name)
        ids = backend.insert(endpoint_name, docs, **kwargs)
        search_backend = self._lookup_backend(endpoint_name)
        if search_backend:
            search_backend.insert(endpoint_name, docs, **kwargs)
        return ids

    def update(self, endpoint_name, id, updates):
        backend = self._backend(endpoint_name)
        res = backend.update(endpoint_name, id, updates)
        search_backend = self._lookup_backend(endpoint_name)
        if search_backend is not None:
            doc = backend.find_one(endpoint_name, req=None, _id=id)
            doc.update(updates)
            search_backend.update(endpoint_name, id, doc)
        return res

    def replace(self, endpoint_name, id, document):
        backend = self._backend(endpoint_name)
        res = backend.replace(endpoint_name, id, document)

        search_backend = self._lookup_backend(endpoint_name)
        if search_backend is not None:
            search_backend.replace(endpoint_name, id, document)
        return res

    def delete(self, endpoint_name, lookup):
        backend = self._backend(endpoint_name)
        res = backend.remove(endpoint_name, lookup)
        search_backend = self._lookup_backend(endpoint_name)
        if search_backend is not None:
            try:
                search_backend.remove(endpoint_name, lookup)
            except ValueError as ex:
                log.error(ex)
                pass
        return res

    def _datasource(self, endpoint_name):
        return app.data._datasource(endpoint_name)[0]

    def _backend(self, endpoint_name):
        return app.data._backend(endpoint_name)

    def _lookup_backend(self, endpoint_name, fallback=False):
        backend = app.data._search_backend(endpoint_name)
        if backend is None and fallback:
            backend = app.data._backend(endpoint_name)
        return backend



"""Superdesk Users"""
from superdesk.resource import Resource


class RolesResource(Resource):
    schema = {
        'name': {
            'type': 'string',
            'unique': True,
            'required': True,
        },
        'description': {
            'type': 'string'
        },
        'extends': {
            'type': 'objectid'
        },
        'permissions': {
            'type': 'dict'
        }
    }
    datasource = {
        'default_sort': [('_created', -1)]
    }


class UsersResource(Resource):
    readonly = False

    additional_lookup = {
        'url': 'regex("[\w]+")',
        'field': 'username'
    }

    schema = {
        'username': {
            'type': 'string',
            'unique': True,
            'required': True,
            'minlength': 1
        },
        'password': {
            'type': 'string',
            'minlength': 5,
            'readonly': readonly
        },
        'first_name': {
            'type': 'string',
            'readonly': readonly
        },
        'last_name': {
            'type': 'string',
            'readonly': readonly
        },
        'display_name': {
            'type': 'string',
            'readonly': readonly
        },
        'email': {
            'unique': True,
            'type': 'email',
            'required': True
        },
        'phone': {
            'type': 'phone_number',
            'readonly': readonly
        },
        'user_info': {
            'type': 'dict'
        },
        'picture_url': {
            'type': 'string',
        },
        'avatar': Resource.rel('upload', True),
        'roles': {
            'type': 'list'
        },
        'preferences': {'type': 'dict'},
        'workspace': {
            'type': 'dict'
        },
        'user_type': {
            'type': 'string',
            'allowed': ['user', 'manager', 'administrator'],
            'default': 'user',
            'required': True
        },
        'status': {
            'type': 'string',
            'allowed': ['active', 'inactive'],
            'default': 'active'
        }
    }

    extra_response_fields = [
        'display_name',
        'username',
        'email',
        'user_info',
        'picture_url',
        'avatar',
        'status'
    ]

    datasource = {
        'projection': {
            'password': 0,
            'preferences': 0
        }
    }



from superdesk.resource import Resource
from superdesk.notification import push_notification
from superdesk.utc import utcnow
from apps.archive.common import base_schema, on_create_item, item_url
from superdesk.services import BaseService
from eve.utils import ParsedRequest
import superdesk


def init_app(app):
    endpoint_name = 'tasks'
    service = TasksService(TaskResource.datasource['source'], backend=superdesk.get_backend())
    TaskResource(endpoint_name, app=app, service=service)


class TaskResource(Resource):
    datasource = {
        'source': 'archive',
        'default_sort': [('_updated', -1)],
        'filter': {'task': {'$exists': True}},
        'elastic_filter': {'exists': {'field': 'task'}}  # eve-elastic specific filter
    }
    item_url = item_url
    schema = {
        'slugline': base_schema['slugline'],
        'description_text': base_schema['description_text'],
        'type': base_schema['type'],
        'planning_item': Resource.rel('planning', True, type='string'),
        'task': {
            'type': 'dict',
            'schema': {
                'status': {
                    'type': 'string',
                    'allowed': ['todo', 'in-progress', 'done'],
                    'default': 'todo'
                },
                'due_date': {'type': 'datetime'},
                'started_at': {'type': 'datetime'},
                'finished_at': {'type': 'datetime'},
                'user': Resource.rel('users', True),
                'desk': Resource.rel('desks', True),
                'stage': Resource.rel('stages', True)
            }
        }
    }


class TasksService(BaseService):

    def get(self, req, lookup):
        if req is None:
            req = ParsedRequest()
        return self.backend.get('tasks', req=req, lookup=lookup)

    def update_times(self, doc):
        task = doc.get('task', {})
        status = task.get('status', None)
        if status == 'in-progress':
            task.setdefault('started_at', utcnow())

        if status == 'done':
            task.setdefault('finished_at', utcnow())

    def update_stage(self, doc):
        task = doc.get('task', {})
        desk_id = task.get('desk', None)
        stage_id = task.get('stage', None)
        if desk_id and not stage_id:
            desk = superdesk.get_resource_service('desks').find_one(req=None, _id=desk_id)
            task['stage'] = desk['incoming_stage']

    def on_create(self, docs):
        on_create_item(docs)
        for doc in docs:
            self.update_times(doc)
            self.update_stage(doc)

    def on_update(self, updates, original):
        self.update_times(updates)

    def on_created(self, docs):
        push_notification(self.datasource, created=1)

    def on_updated(self, updates, original):
        push_notification(self.datasource, updated=1)

    def on_deleted(self, doc):
        push_notification(self.datasource, deleted=1)

    def assign_user(self, item_id, user):
        item = self.find_one(req=None, _id=item_id)
        item['task'] = item.get('task', {})
        item['task']['user'] = user
        del item['_id']
        return self.update(item_id, item)



import os
import unittest
from superdesk.io.newsml_1_2 import Parser
from superdesk.etree import etree
import datetime


class TestCase(unittest.TestCase):
    def setUp(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.join(dirname, 'fixtures', 'afp.xml')
        with open(fixture) as f:
            self.item = Parser().parse_message(etree.fromstring(f.read()))

    def test_headline(self):
        self.assertEquals(self.item.get('headline'), 'Sweden court accepts receivership for Saab carmaker')

    def test_guid(self):
        self.assertEquals(self.item.get('guid'), 'urn:newsml:afp.com:20140829T135002Z:TX-PAR-FXW86:1')

    def test_coreitemvalues(self):
        self.assertEquals(self.item.get('provider'), 'AFP')
        self.assertEquals(self.item.get('type'), 'text')
        self.assertEquals(self.item.get('urgency'), '4')
        self.assertEquals(self.item.get('version'), '1')
        self.assertEquals(self.item.get('versioncreated'), datetime.datetime(2014, 8, 29, 13, 49, 51))
        self.assertEquals(self.item.get('firstcreated'), datetime.datetime(2014, 8, 29, 13, 49, 51))
        self.assertEquals(self.item.get('pubstatus'), 'Usable')

    def test_subjects(self):
        self.assertEquals(len(self.item.get('subject')), 8)
        self.assertIn({'cat': 'ECO', 'FormalName': '04011002'}, self.item.get('subject'))
        self.assertIn({'cat': 'ECO', 'FormalName': '04016007'}, self.item.get('subject'))
        self.assertIn({'cat': 'ECO', 'FormalName': '04000000'}, self.item.get('subject'))
        self.assertIn({'cat': 'ECO', 'FormalName': '04016038'}, self.item.get('subject'))
        self.assertIn({'cat': 'ECO', 'FormalName': '04011000'}, self.item.get('subject'))

if __name__ == '__main__':
    unittest.main()



from apps.auth.auth import SuperdeskTokenAuth
from .auth import AuthUsersResource, AuthResource  # noqa
from .sessions import SesssionsResource
import superdesk
from superdesk.services import BaseService
from .db.reset_password import reset_schema  # noqa


def init_app(app):
    app.auth = SuperdeskTokenAuth()  # Overwrite the app default auth

    endpoint_name = 'auth_users'
    service = BaseService(endpoint_name, backend=superdesk.get_backend())
    AuthUsersResource(endpoint_name, app=app, service=service)

    endpoint_name = 'sessions'
    service = BaseService(endpoint_name, backend=superdesk.get_backend())
    SesssionsResource(endpoint_name, app=app, service=service)



"""Reuters io service."""

import requests
import traceback
import datetime
from superdesk import get_resource_service

from superdesk.utc import utcnow
from superdesk.utc import utc
from superdesk.etree import etree
from urllib.parse import urlparse, urlunparse
from superdesk.io import register_provider, IngestService
from .newsml_2_0 import Parser
from .reuters_token import get_token


PROVIDER = 'reuters'


def _on_read_ingest(docs):
    provider = get_resource_service('ingest_providers').find_one(type=PROVIDER, req=None)
    if not provider:
        return

    for doc in docs['_items']:
        if str(doc.get('ingest_provider')) == str(provider['_id']):
            for i, rendition in doc.get('renditions', {}).items():
                rendition['href'] = '%s?auth_token=%s' % (rendition['href'], get_token(provider))


def init_app(app):
    app.on_fetched_resource_ingest -= _on_read_ingest
    app.on_fetched_resource_ingest += _on_read_ingest


class ReutersIngestService(IngestService):
    """Reuters ingest service."""

    DATE_FORMAT = '%Y.%m.%d.%H.%M'
    URL = 'http://rmb.reuters.com/rmd/rest/xml'
    token = None

    def __init__(self):
        self.parser = Parser()

    def get_token(self):
        """Get reuters token once for an update run."""

        if not self.token:
            self.token = get_token(self.provider, update=True)
        return self.token

    def update(self, provider):
        """Service update call."""

        self.provider = provider
        updated = utcnow()

        last_updated = provider.get('_updated')
        if not last_updated or last_updated < updated + datetime.timedelta(days=-7):
            last_updated = updated + datetime.timedelta(hours=-24)

        for channel in self.get_channels():
            for guid in self.get_ids(channel, last_updated, updated):
                items = self.fetch_ingest(guid)
                yield items

    def fetch_ingest(self, guid):
        items = self.get_items(guid)
        result_items = []
        while items:
            item = items.pop()
            result_items.append(item)
            item['_created'] = item['firstcreated'] = utc.localize(item['firstcreated'])
            item['_updated'] = item['versioncreated'] = utc.localize(item['versioncreated'])
            items.extend(self.fetch_assets(item))
        return result_items

    def fetch_assets(self, item):
        """Fetch remote assets for given item."""
        items = []
        for group in item.get('groups', []):
            for ref in group.get('refs', []):
                if 'residRef' in ref:
                    items.extend(self.get_items(ref.get('residRef')))
        return items

    def get_items(self, guid):
        """Parse item message and return given items."""
        payload = {'id': guid}
        tree = self.get_tree('item', payload)
        items = self.parser.parse_message(tree)
        return items

    def get_ids(self, channel, last_updated, updated):
        """Get ids of documents which should be updated."""

        ids = []
        payload = {'channel': channel, 'fieldsRef': 'id'}
        payload['dateRange'] = "%s-%s" % (self.format_date(last_updated), self.format_date(updated))
        tree = self.get_tree('items', payload)
        for result in tree.findall('result'):
            ids.append(result.find('guid').text)
        return ids

    def get_channels(self):
        """Get subscribed channels."""

        channels = []
        tree = self.get_tree('channels')
        for channel in tree.findall('channelInformation'):
            channels.append(channel.find('alias').text)
        return channels

    def get_tree(self, endpoint, payload=None):
        """Get xml response for given API endpoint and payload."""

        if payload is None:
            payload = {}
        payload['token'] = self.get_token()
        url = self.get_url(endpoint)

        try:
            response = requests.get(url, params=payload, timeout=21.0)
        except Exception as error:
            traceback.print_exc()
            raise error

        if response.status_code == 404:
            raise LookupError('Not found %s' % payload)

        try:
            # workaround for httmock lib
            # return etree.fromstring(response.text.encode('utf-8'))
            return etree.fromstring(response.content)
        except UnicodeEncodeError as error:
            traceback.print_exc()
            raise error

    def get_url(self, endpoint):
        """Get API url for given endpoint."""
        return '/'.join([self.URL, endpoint])

    def format_date(self, date):
        """Format date for API usage."""
        return date.strftime(self.DATE_FORMAT)

    def prepare_href(self, href):
        (scheme, netloc, path, params, query, fragment) = urlparse(href)
        new_href = urlunparse((scheme, netloc, path, '', '', ''))
        return '%s?auth_token=%s' % (new_href, self.get_token())


register_provider(PROVIDER, ReutersIngestService())



from superdesk import utils as utils
from apps.auth.errors import UserInactiveError
from superdesk.services import BaseService


class AuthService(BaseService):

    def authenticate(self, document):
        pass

    def on_create(self, docs):
        for doc in docs:
            user = self.authenticate(doc)

            if user.get('status', 'active') == 'inactive':
                raise UserInactiveError()

            doc['user'] = user['_id']
            doc['token'] = utils.get_random_string(40)
            del doc['password']




import os
from datetime import timedelta
from superdesk.tests import TestCase
import superdesk
from superdesk.utc import utcnow
from .reuters_token import get_token
from .reuters import PROVIDER


def setup_provider(token, hours):
    return {
        'name': PROVIDER,
        'type': PROVIDER,
        'token': {
            'token': token,
            'created': utcnow() - timedelta(hours=hours),
        }
    }


class GetTokenTestCase(TestCase):

    def test_get_null_token(self):
        provider = {}
        self.assertEquals('', get_token(provider))

    def test_get_existing_token(self):
        provider = setup_provider('abc', 10)
        self.assertEquals('abc', get_token(provider))

    def test_get_expired_token(self):
        """Expired is better than none.."""
        provider = setup_provider('abc', 24)
        self.assertEquals('abc', get_token(provider))

    def test_fetch_token(self):
        with self.app.test_request_context():
            provider = setup_provider('abc', 24)
            superdesk.get_resource_service('ingest_providers').post([provider])
            self.assertTrue(provider.get('_id'))
            provider['config'] = {}
            provider['config']['username'] = os.environ.get('REUTERS_USERNAME', '')
            provider['config']['password'] = os.environ.get('REUTERS_PASSWORD', '')
            if provider['config']['username']:
                token = get_token(provider, update=True)
                self.assertNotEquals('', token)
                self.assertEquals(token, provider['token']['token'])

                dbprovider = superdesk.get_resource_service('ingest_providers').find_one(type=PROVIDER, req=None)
                self.assertEquals(token, dbprovider['token']['token'])



from superdesk import app_components
from .base_component import BaseComponent


def register_component(component):
    '''
    Register a component - to be used in bootstrap.
    @param component: object
        The component object.
    '''
    assert isinstance(component, BaseComponent), 'Invalid component %s' % component
    app_components[component.name()] = component


def get_component(name):
    '''
    Return an instance of the component identified by given name.
    '''
    if isinstance(name, BaseComponent) or (type(name) is type and issubclass(name, BaseComponent)):
        name = name.name()
    return app_components[name]



import os
from datetime import timedelta
import json

try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse

XML = False
IF_MATCH = True
BANDWIDTH_SAVER = False
DATE_FORMAT = '%Y-%m-%dT%H:%M:%S+0000'

server_url = urlparse(os.environ.get('SUPERDESK_URL', 'http://localhost:5000'))
URL_PROTOCOL = server_url.scheme or None
SERVER_NAME = server_url.netloc or None
URL_PREFIX = server_url.path.lstrip('/') or ''
VALIDATION_ERROR_STATUS = 400

CACHE_CONTROL = 'max-age=0, no-cache'

X_DOMAINS = '*'
X_MAX_AGE = 24 * 3600
X_HEADERS = ['Content-Type', 'Authorization', 'If-Match']


MONGO_DBNAME = os.environ.get('MONGO_DBNAME', 'superdesk')
if os.environ.get('MONGOLAB_URI'):
    MONGO_URI = os.environ.get('MONGOLAB_URI')
elif os.environ.get('MONGODB_PORT'):
    MONGO_URI = '{0}/{1}'.format(os.environ.get('MONGODB_PORT').replace('tcp:', 'mongodb:'), MONGO_DBNAME)

ELASTICSEARCH_URL = os.environ.get('ELASTICSEARCH_URL', 'http://localhost:9200')
ELASTICSEARCH_INDEX = os.environ.get('ELASTICSEARCH_INDEX', 'superdesk')
if os.environ.get('ELASTIC_PORT'):
    ELASTICSEARCH_URL = os.environ.get('ELASTIC_PORT').replace('tcp:', 'http:')

REDIS_URL = os.environ.get('REDIS_URL', 'redis://localhost:6379')
if os.environ.get('REDIS_PORT'):
    REDIS_URL = os.environ.get('REDIS_PORT').replace('tcp:', 'redis:')
BROKER_URL = os.environ.get('CELERY_BROKER_URL', REDIS_URL)
CELERY_RESULT_BACKEND = os.environ.get('CELERY_RESULT_BACKEND', REDIS_URL)
CELERY_ALWAYS_EAGER = (os.environ.get('CELERY_ALWAYS_EAGER', False) == 'True')
CELERY_TASK_SERIALIZER = 'json'
CELERY_ACCEPT_CONTENT = ['pickle', 'json']  # it's using pickle when in eager mode

CELERYBEAT_SCHEDULE = {
    'fetch_ingest': {
        'task': 'superdesk.io.fetch_ingest',
        'schedule': timedelta(minutes=5)
    }
}

SENTRY_DSN = os.environ.get('SENTRY_DSN')
SENTRY_INCLUDE_PATHS = ['superdesk']

INSTALLED_APPS = [
    'apps.auth',
    'apps.users',
    'superdesk.upload',
    'superdesk.notification',
    'superdesk.activity',
    'superdesk.comments',
    'superdesk.storage.amazon.import_from_amazon',

    'superdesk.io',
    'superdesk.io.subjectcodes',
    'superdesk.io.reuters',
    'superdesk.io.aap',
    'superdesk.io.afp',

    'apps.archive',
    'apps.stages',
    'apps.desks',
    'apps.planning',
    'apps.coverages',
    'apps.tasks',
    'apps.preferences'
]

RESOURCE_METHODS = ['GET', 'POST']
ITEM_METHODS = ['GET', 'PATCH', 'PUT', 'DELETE']
EXTENDED_MEDIA_INFO = ['content_type', 'name', 'length']
RETURN_MEDIA_AS_BASE64_STRING = False

AMAZON_CONTAINER_NAME = os.environ.get('AMAZON_CONTAINER_NAME', '')
AMAZON_ACCESS_KEY_ID = os.environ.get('AMAZON_ACCESS_KEY_ID', '')
AMAZON_SECRET_ACCESS_KEY = os.environ.get('AMAZON_SECRET_ACCESS_KEY', '')
AMAZON_REGION = os.environ.get('AMAZON_REGION', '')


RENDITIONS = {
    'picture': {
        'thumbnail': {'width': 220, 'height': 120},
        'viewImage': {'width': 640, 'height': 640},
        'baseImage': {'width': 1400, 'height': 1400},
    },
    'avatar': {
        'thumbnail': {'width': 60, 'height': 60},
        'viewImage': {'width': 200, 'height': 200},
    }
}

SERVER_DOMAIN = 'localhost'

BCRYPT_GENSALT_WORK_FACTOR = 12
RESET_PASSWORD_TOKEN_TIME_TO_LIVE = int(os.environ.get('RESET_PASS_TTL', 24))  # The number of hours a token is valid

# email server
MAIL_SERVER = os.environ.get('MAIL_SERVER', 'smtp.googlemail.com')
MAIL_PORT = int(os.environ.get('MAIL_PORT', 465))
MAIL_USE_TLS = json.loads(os.environ.get('MAIL_USE_TLS', 'False').lower())
MAIL_USE_SSL = json.loads(os.environ.get('MAIL_USE_SSL', 'True').lower())
MAIL_USERNAME = os.environ.get('MAIL_USERNAME', 'admin@sourcefabric.org')
MAIL_PASSWORD = os.environ.get('MAIL_PASSWORD', 'admin-password')
ADMINS = [MAIL_USERNAME]

# LDAP settings
LDAP_SERVER = os.environ.get('LDAP_SERVER', '')  # Ex: ldap://sourcefabric.org
LDAP_SERVER_PORT = os.environ.get('LDAP_SERVER_PORT', 389)

# Fully Qualified Domain Name. Ex: sourcefabric.org
LDAP_FQDN = os.environ.get('LDAP_FQDN', '')

# LDAP_BASE_FILTER limit the base filter to the security group. Ex: OU=Superdesk Users,dc=sourcefabric,dc=org
LDAP_BASE_FILTER = os.environ.get('LDAP_BASE_FILTER', '')

# change the user depending on the LDAP directory structure
LDAP_USER_FILTER = os.environ.get('LDAP_USER_FILTER', "(&(objectCategory=user)(objectClass=user)(sAMAccountName={}))")

# LDAP User Attributes to fetch. Keys would be LDAP Attribute Name and Value would be Supderdesk Model Attribute Name
LDAP_USER_ATTRIBUTES = {'givenName': 'first_name', 'sn': 'last_name', 'displayName': 'display_name',
                        'mail': 'email', 'ipPhone': 'phone'}

if LDAP_SERVER:
    INSTALLED_APPS.append('apps.auth.ldap')
else:
    INSTALLED_APPS.append('apps.auth.db')




import os
from datetime import datetime, timedelta

from .nitf import parse
from superdesk.utc import utc, utcnow, timezone
from superdesk.io import register_provider, IngestService

PROVIDER = 'aap'


def is_ready(last_updated, provider_last_updated=None):
    """Parse file only if it's not older than provider last update -10m"""

    if not provider_last_updated:
        provider_last_updated = utcnow() - timedelta(days=7)

    return provider_last_updated - timedelta(minutes=10) < last_updated


def normalize_date(naive, tz):
    return utc.normalize(tz.localize(naive))


class AAPIngestService(IngestService):
    """AAP Ingest Service"""

    def __init__(self):
        self.tz = timezone('Australia/Sydney')

    def prepare_href(self, href):
        return href

    def update(self, provider):
        path = provider.get('config', {}).get('path', None)
        if not path:
            return

        for filename in os.listdir(path):
            filepath = os.path.join(path, filename)
            stat = os.lstat(filepath)
            last_updated = datetime.fromtimestamp(stat.st_mtime, tz=utc)
            if is_ready(last_updated, provider.get('updated')):
                with open(os.path.join(path, filename), 'r') as f:
                    item = parse(f.read())
                    item['firstcreated'] = normalize_date(item.get('firstcreated'), self.tz)
                    item['versioncreated'] = normalize_date(item.get('versioncreated'), self.tz)
                    item['created'] = item['firstcreated']
                    item['updated'] = item['versioncreated']
                    item.setdefault('provider', provider.get('name', provider['type']))
                    yield [item]

register_provider(PROVIDER, AAPIngestService())



from superdesk.resource import Resource
from .common import base_schema, extra_response_fields, item_url, facets
from .common import on_create_item, on_create_media_archive, on_update_media_archive, on_delete_media_archive
from superdesk.services import BaseService


class IngestResource(Resource):
    schema = {
        'archived': {
            'type': 'datetime'
        }
    }
    schema.update(base_schema)
    extra_response_fields = extra_response_fields
    item_url = item_url
    datasource = {
        'search_backend': 'elastic',
        'facets': facets
    }


class IngestService(BaseService):

    def on_create(self, docs):
        on_create_item(docs)
        on_create_media_archive()

    def on_update(self, updates, original):
        on_update_media_archive()

    def on_delete(self, doc):
        on_delete_media_archive()



'''
Utilities for extractid metadata from image files.
'''

from PIL import Image, ExifTags
from flask import json


def get_meta(file_stream):
    '''
    Returns the image metadata in a dictionary of tag:value pairs.

    @param file_stream: stream
    '''
    current = file_stream.tell()
    file_stream.seek(0)
    img = Image.open(file_stream)
    if not hasattr(img, '_getexif'):
        return {}
    rv = img._getexif()
    if not rv:
        return {}
    exif = dict(rv)
    file_stream.seek(current)

    exif_meta = {}
    for k, v in exif.items():
        try:
            json.dumps(v)
            value = v.decode('UTF-8') if isinstance(v, bytes) else v
            exif_meta[ExifTags.TAGS[k].strip()] = value
        except:
            # ignore fields we can't store in db
            pass
    # Remove this as it's too long to send in headers
    if exif_meta.get('UserComment'):
        del exif_meta['UserComment']
    return exif_meta



import logging
from flask import current_app as app
from eve.defaults import resolve_default_values
from eve.utils import ParsedRequest, config
from eve.methods.common import resolve_document_etag


log = logging.getLogger(__name__)


class BaseService():
    '''
    Base service for all endpoints, defines the basic implementation
    for CRUD datalayer functionality.
    '''
    datasource = None

    def __init__(self, datasource, backend):
        self.backend = backend
        self.datasource = datasource

    def on_create(self, docs):
        pass

    def on_created(self, docs):
        pass

    def on_update(self, updates, original):
        pass

    def on_updated(self, updates, original):
        pass

    def on_replace(self, document, original):
        pass

    def on_replaced(self, document, original):
        pass

    def on_delete(self, doc):
        pass

    def on_deleted(self, doc):
        pass

    def on_fetched(self, doc):
        pass

    def create(self, docs, **kwargs):
        ids = self.backend.create(self.datasource, docs, **kwargs)
        return ids

    def update(self, id, updates):
        res = self.backend.update(self.datasource, id, updates)
        return res

    def replace(self, id, document):
        res = self.backend.replace(self.datasource, id, document)
        return res

    def delete(self, lookup):
        res = self.backend.delete(self.datasource, lookup)
        return res

    def find_one(self, req, **lookup):
        res = self.backend.find_one(self.datasource, req=req, **lookup)
        return res

    def get(self, req, lookup):
        if req is None:
            req = ParsedRequest()
        return self.backend.get(self.datasource, req=req, lookup=lookup)

    def post(self, docs, **kwargs):
        for doc in docs:
            resolve_default_values(doc, app.config['DOMAIN'][self.datasource]['defaults'])
        self.on_create(docs)
        resolve_document_etag(docs)
        ids = self.create(docs, **kwargs)
        self.on_created(docs)
        return ids

    def patch(self, id, updates):
        original = self.find_one(req=None, _id=id)
        updated = original.copy()
        self.on_update(updates, original)
        updated.update(updates)
        if config.IF_MATCH:
            resolve_document_etag(updated)
            updates[config.ETAG] = updated[config.ETAG]
        res = self.update(id, updates)
        self.on_updated(updates, original)
        return res

    def put(self, id, document):
        resolve_default_values(document, app.config['DOMAIN'][self.datasource]['defaults'])
        original = self.find_one(req=None, _id=id)
        self.on_replace(document, original)
        resolve_document_etag(document)
        res = self.replace(id, document)
        self.on_replaced(document, original)
        return res

    def delete_action(self, lookup=None):
        if lookup is None:
            lookup = {}

        if lookup:
            doc = self.find_one(req=None, **lookup)
            self.on_delete(doc)
        res = self.delete(lookup)

        if lookup and doc:
            self.on_deleted(doc)
        return res



import logging
from superdesk.resource import Resource
from superdesk.services import BaseService
import superdesk


logger = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = 'stages'
    service = StagesService(endpoint_name, backend=superdesk.get_backend())
    StagesResource(endpoint_name, app=app, service=service)


class StagesResource(Resource):
    schema = {
        'name': {
            'type': 'string',
            'required': True,
            'minlength': 1
        },
        'description': {
            'type': 'string',
            'minlength': 1
        },
        'default_incoming': {
            'type': 'boolean',
            'required': True,
            'default': False
        },
        'desk': Resource.rel('desks', embeddable=True),
        'outgoing': {
            'type': 'list',
            'schema': {
                'type': 'dict',
                'schema': {
                    'stage': Resource.rel('stages', True)
                }
            }
        }
    }


class StagesService(BaseService):

    def on_delete(self, doc):
        if doc['default_incoming'] is True:
            desk_id = doc.get('desk', None)
            if desk_id:
                desk = superdesk.get_resource_service('desks').find_one(req=None, _id=desk_id)
                if desk:
                    raise superdesk.SuperdeskError('Deleting default stages is not allowed.')




import os
from superdesk.tests import TestCase
from .image import get_meta


class ExifMetaExtractionTestCase(TestCase):

    fixtures = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'fixtures')
    img = os.path.join(fixtures, 'canon_exif.JPG')

    def test_extract_meta_json_serialization(self):

        with open(self.img, mode='rb') as f:
            meta = get_meta(f)

        self.assertEquals(meta['ExifImageWidth'], 32)
        self.assertEquals(meta['ExifImageHeight'], 21)
        self.assertEquals(meta['Make'], 'Canon')
        self.assertEquals(meta['Model'], 'Canon EOS 60D')



from apps.common.models.base_model import BaseModel
from apps.item_lock.models.item import ItemValidator


class ItemAutosaveModel(BaseModel):
    def __init__(self, data_layer):
        BaseModel.__init__(self, 'archive_autosave', data_layer, {}, ItemValidator())

    @classmethod
    def name(cls):
        return 'item_autosave'



from .data_layer import DataLayer
from eve.utils import ParsedRequest, config, document_etag
from eve import ID_FIELD


class EveProxy(DataLayer):
    '''
    Data layer implementation used to connect the models to the Eve data layer.
    Transforms the model data layer API into Eve data layer calls.
    '''
    def __init__(self, eve_data_layer):
        self.eve_data_layer = eve_data_layer

    def etag(self, doc):
        return doc.get(config.ETAG, document_etag(doc))

    def find_one(self, resource, filter, projection):
        req = ParsedRequest()
        req.args = {}
        req.projection = projection
        return self.eve_data_layer.find_one(resource, req, **filter)

    def find(self, resource, filter, projection, **options):
        req = ParsedRequest()
        req.args = {}
        req.projection = projection
        return self.eve_data_layer.get(resource, req, filter)

    def create(self, resource, docs):
        return self.eve_data_layer.create(resource, docs)

    def update(self, resource, filter, doc):
        return self.eve_data_layer.update(resource, filter[ID_FIELD], doc)

    def replace(self, resource, filter, doc):
        return self.eve_data_layer.replace(resource, filter[ID_FIELD], doc)

    def delete(self, resource, filter):
        return self.eve_data_layer.delete(resource, filter)




from .tests import TestCase
import superdesk


class DatalayerTestCase(TestCase):

    def test_find_all(self):
        data = {'resource': 'test', 'action': 'get'}
        with self.app.test_request_context():
            superdesk.get_resource_service('activity').post([data])
            self.assertEquals(1, superdesk.get_resource_service('activity').get(req=None, lookup={}).count())



from flask import json
from datetime import datetime


class ClientMock:

    def __init__(self):
        self.messages = []
        self.client = None

    def notify(self, **kwargs):
        kwargs.setdefault('_created', datetime.utcnow().isoformat())
        self.messages.append(json.dumps(kwargs).encode('utf8'))


def setup_notification_mock(context):
    clientMock = ClientMock()
    if context.app.notification_client:
        clientMock.client = context.app.notification_client
    context.app.notification_client = clientMock


def teardown_notification_mock(context):
    context.app.notification_client = context.app.notification_client.client



from .users import RolesResource, UsersResource
from .services import ADUsersService, DBUsersService
import superdesk
from superdesk.services import BaseService


def init_app(app):

    is_ldap = bool(app.config.get('LDAP_SERVER', False))

    endpoint_name = 'users'
    if is_ldap:
        service = ADUsersService(endpoint_name, backend=superdesk.get_backend())
    else:
        service = DBUsersService(endpoint_name, backend=superdesk.get_backend())

    UsersResource.readonly = is_ldap
    UsersResource(endpoint_name, app=app, service=service)

    endpoint_name = 'roles'
    service = BaseService(endpoint_name, backend=superdesk.get_backend())
    RolesResource(endpoint_name, app=app, service=service)



from flask.ext.mail import Message
from superdesk.celery_app import celery
from flask import current_app as app


@celery.task(bind=True, max_retries=3)
def send_email(self, subject, sender, recipients, text_body, html_body):
    msg = Message(subject, sender=sender, recipients=recipients)
    msg.body = text_body
    msg.html = html_body
    return app.mail.send(msg)



from ..models.item import ItemModel
from superdesk import SuperdeskError
from superdesk.utc import utcnow
from superdesk.notification import push_notification
from apps.common.components.base_component import BaseComponent
from apps.common.models.utils import get_model
import superdesk


LOCK_USER = 'lock_user'
STATUS = '_status'


class ItemLock(BaseComponent):
    def __init__(self, app):
        self.app = app

    @classmethod
    def name(cls):
        return 'item_lock'

    def lock(self, filter, user, etag):
        item_model = get_model(ItemModel)
        item = item_model.find_one(filter)
        if item and self._can_lock(item, user):
            self.app.on_item_lock(item, user)
            updates = {LOCK_USER: user, 'lock_time': utcnow()}
            item_model.update(filter, updates)
            superdesk.get_resource_service('tasks').assign_user(item['_id'], user)
            self.app.on_item_locked(item, user)
            push_notification('item:lock', item=str(item.get('_id')), user=str(user))
        else:
            raise SuperdeskError('Item locked by another user')
        return item

    def unlock(self, filter, user, etag):
        item_model = get_model(ItemModel)
        item = item_model.find_one(filter)
        if item:
            self.app.on_item_unlock(item, user)
            updates = {LOCK_USER: None, 'lock_time': None, 'force_unlock': True}
            item_model.update(filter, updates)
            self.app.on_item_unlocked(item, user)
            push_notification('item:unlock', item=str(filter.get('_id')))

    def _can_lock(self, item, user):
        # TODO: implement
        return True




from superdesk.resource import Resource, build_custom_hateoas
from superdesk.services import BaseService
from .archive import ArchiveResource


class UserContentResource(Resource):
    endpoint_name = 'user_content'
    item_url = ArchiveResource.item_url
    url = 'users/<regex("[a-f0-9]{24}"):original_creator>/content'
    schema = ArchiveResource.schema
    datasource = {'source': 'archive'}
    resource_methods = ['GET', 'POST']
    item_methods = ['GET', 'PATCH', 'DELETE']
    resource_title = endpoint_name


class UserContentService(BaseService):
    custom_hateoas = {'self': {'title': 'Archive', 'href': '/archive/{_id}'}}

    def get(self, req, lookup):
        docs = super().get(req, lookup)
        for doc in docs:
            build_custom_hateoas(self.custom_hateoas, doc)
        return docs



"""Generic comments module."""
from .comments import CommentsService, CommentsResource, comments_schema  # noqa
import superdesk


def init_app(app):
    endpoint_name = 'comments'
    service = CommentsService(endpoint_name, backend=superdesk.get_backend())
    CommentsResource(endpoint_name, app=app, service=service)




import os
import json
import logging
import asyncio
import signal
from autobahn.asyncio.websocket import WebSocketServerProtocol
from autobahn.asyncio.websocket import WebSocketServerFactory


host = '0.0.0.0'
beat_delay = 30
port = int(os.environ.get('WSPORT', '5100'))
logger = logging.getLogger(__name__)


def log(msg):
    log_msg = msg
    logger.info(log_msg)
    print(log_msg)


class BroadcastProtocol(WebSocketServerProtocol):
    """Client protocol - there is an instance per client connection."""

    def onOpen(self):
        """Register"""
        self.factory.register(self)

    def onMessage(self, payload, isBinary):
        """Broadcast msg from a client"""
        self.factory.broadcast(payload, self.peer)

    def connectionLost(self, reason):
        """Unregister on connection drop"""
        WebSocketServerProtocol.connectionLost(self, reason)
        self.factory.unregister(self)


class BroadcastServerFactory(WebSocketServerFactory):
    """Server handling client registrations and broadcasting"""

    def __init__(self, *args):
        WebSocketServerFactory.__init__(self, *args)
        self.clients = []

    def register(self, client):
        """Register a client"""
        if client not in self.clients:
            log('registered client {}'.format(client.peer))
            self.clients.append(client)
            client.sendMessage(json.dumps({'event': 'connected'}).encode('utf8'))

    def unregister(self, client):
        """Unregister a client"""
        if client in self.clients:
            log('unregister client {}'.format(client.peer))
            self.clients.remove(client)

    def broadcast(self, msg, author):
        """Broadcast msg to all clients but author."""
        log('broadcasting "{}"'.format(msg.decode('utf8')))

        for c in self.clients:
            if c.state == c.STATE_CLOSED:
                self.unregister(c)

        for c in self.clients:
            if c.peer is not author:
                c.sendMessage(msg)
        log('msg sent to {0} client(s)'.format(len(self.clients) - 1))


def send_heartbeat(server, loop):
    yield from asyncio.sleep(beat_delay)
    while loop.is_running():
        server.broadcast(json.dumps({'heartbeat': 'ping'}).encode('utf8'), None)
        yield from asyncio.sleep(beat_delay)


if __name__ == '__main__':

    factory = BroadcastServerFactory()
    factory.protocol = BroadcastProtocol

    loop = asyncio.get_event_loop()
    coro = loop.create_server(factory, host, port)
    server = loop.run_until_complete(coro)

    def stop():
        log('closing...')
        server.close()
        loop.call_soon_threadsafe(loop.stop)

    loop.add_signal_handler(signal.SIGTERM, stop)

    try:
        log('initializing heartbeat...')
        asyncio.async(send_heartbeat(factory, loop))

        log('listening on {0}:{1}'.format(host, port))
        loop.run_forever()
    except KeyboardInterrupt:
        pass
    finally:
        stop()



"""Superdesk Manager"""

import superdesk
from flask.ext.script import Manager
from app import get_app

app = get_app()
manager = Manager(app)

if __name__ == '__main__':
    manager.run(superdesk.COMMANDS)





from superdesk.tests import TestCase
from superdesk.celery_app import try_cast, loads
from bson import ObjectId
from datetime import datetime
from eve.utils import date_to_str


class CeleryTestCase(TestCase):

    _id = ObjectId('528de7b03b80a13eefc5e610')

    def test_cast_objectid(self):
        self.assertEquals(try_cast(str(self._id)), self._id)

    def test_cast_datetime(self):
        date = datetime(2012, 12, 12, 12, 12, 12, 0)
        with self.app.app_context():
            s = date_to_str(date)
            self.assertEquals(try_cast(s).day, date.day)

    def test_loads_args(self):
        s = b'{"args": [{"_id": "528de7b03b80a13eefc5e610", "_updated": "2014-09-10T14:31:09+0000"}]}'
        o = loads(s)
        self.assertEquals(o['args'][0]['_id'], self._id)
        self.assertIsInstance(o['args'][0]['_updated'], datetime)



'''Media archive module'''
from .archive import ArchiveResource, ArchiveService, ArchiveVersionsResource, AutoSaveResource, \
    ArchiveVersionsService, ArchiveSaveService
from .ingest import IngestResource, IngestService
from .archive_media import ArchiveMediaResource, ArchiveMediaService
from .archive_ingest import ArchiveIngestResource, ArchiveIngestService
from .item_comments import ItemCommentsResource, ItemCommentsSubResource, ItemCommentsService, ItemCommentsSubService
from .user_content import UserContentResource, UserContentService
from .archive_lock import ArchiveLockResource, ArchiveUnlockResource, ArchiveLockService, ArchiveUnlockService
from .content_view import ContentViewResource, ContentViewItemsResource, ContentViewService, ContentViewItemsService
import superdesk
from apps.common.components.utils import register_component
from apps.item_lock.components.item_lock import ItemLock
from apps.common.models.utils import register_model
from apps.item_lock.models.item import ItemModel
from apps.common.models.io.eve_proxy import EveProxy


def init_app(app):

    endpoint_name = 'ingest'
    service = IngestService(endpoint_name, backend=superdesk.get_backend())
    IngestResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive_versions'
    service = ArchiveVersionsService(endpoint_name, backend=superdesk.get_backend())
    ArchiveVersionsResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive'
    service = ArchiveService(endpoint_name, backend=superdesk.get_backend())
    ArchiveResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive_media'
    service = ArchiveMediaService(endpoint_name, backend=superdesk.get_backend())
    ArchiveMediaResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive_ingest'
    service = ArchiveIngestService(endpoint_name, backend=superdesk.get_backend())
    ArchiveIngestResource(endpoint_name, app=app, service=service)

    endpoint_name = 'item_comments'
    service = ItemCommentsService(endpoint_name, backend=superdesk.get_backend())
    ItemCommentsResource(endpoint_name, app=app, service=service)

    endpoint_name = 'content_item_comments'
    service = ItemCommentsSubService(endpoint_name, backend=superdesk.get_backend())
    ItemCommentsSubResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive_lock'
    service = ArchiveLockService(endpoint_name, backend=superdesk.get_backend())
    ArchiveLockResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive_unlock'
    service = ArchiveUnlockService(endpoint_name, backend=superdesk.get_backend())
    ArchiveUnlockResource(endpoint_name, app=app, service=service)

    endpoint_name = 'user_content'
    service = UserContentService(endpoint_name, backend=superdesk.get_backend())
    UserContentResource(endpoint_name, app=app, service=service)

    endpoint_name = 'content_view'
    service = ContentViewService(endpoint_name, backend=superdesk.get_backend())
    ContentViewResource(endpoint_name, app=app, service=service)

    endpoint_name = 'content_view_items'
    service = ContentViewItemsService(endpoint_name, backend=superdesk.get_backend())
    ContentViewItemsResource(endpoint_name, app=app, service=service)

    endpoint_name = 'archive_autosave'
    service = ArchiveSaveService(endpoint_name, backend=superdesk.get_backend())
    AutoSaveResource(endpoint_name, app=app, service=service)

    from apps.item_autosave.components.item_autosave import ItemAutosave
    from apps.item_autosave.models.item_autosave import ItemAutosaveModel
    register_component(ItemLock(app))
    register_model(ItemModel(EveProxy(superdesk.get_backend())))
    register_component(ItemAutosave(app))
    register_model(ItemAutosaveModel(EveProxy(superdesk.get_backend())))



def json_get_value(json, path):
    '''
    Get the value from specified json path
    json: json document
    path: a list of keys
    '''
    crt = json
    for name in path:
        if name in crt:
            crt = crt[name]
        else:
            return None
    return crt


def json_set_value(json, path, value):
    '''
    Set the value at the specified json path.
    Create the path if it don't exists.
    json: json document
    path: a list of keys
    value: value to be set
    '''
    crt = json
    for name in path[:-1]:
        if name not in crt:
            crt[name] = {}
        crt = crt[name]
    crt[path[-1]] = value


def json_merge_values(destination, source, path, merge_values_mode):
    '''
    On destination json merge the source value defined by path.
    The values are merged by calling the merge_values_mode method
    destination: destination json
    source: source json
    path: list of keys
    merge_values_mode: a method that will get the values as parameters
    and will return the merged value
    '''
    destination_term = json_get_value(destination, path)
    source_term = json_get_value(source, path)

    if not source_term:
        return

    if destination_term:
        json_set_value(destination, path, merge_values_mode(destination_term, source_term))
    else:
        json_set_value(destination, path, source_term)


def json_copy_values(destination, source, keys):
    '''
    If a key is not present on destination it will be copied from source (if exists)
    destination - destination json
    source - source json
    keys - list of keys
    '''
    for key in keys:
        if key not in destination and key in source:
            destination[key] = source[key]




import logging
import superdesk
from eve.utils import config


log = logging.getLogger(__name__)


def build_custom_hateoas(hateoas, doc, **values):
    values.update(doc)
    links = doc.get(config.LINKS)
    if not links:
        links = {}
        doc[config.LINKS] = links

    for link_name in hateoas.keys():
        link = hateoas[link_name]
        link = {'title': link['title'], 'href': link['href']}
        link['href'] = link['href'].format(**values)
        links[link_name] = link


class Resource():
    '''
    Base model for all endpoints, defines the basic implementation
    for CRUD datalayer functionality.
    '''
    endpoint_name = None
    url = None
    item_url = None
    additional_lookup = None
    schema = {}
    item_methods = None
    resource_methods = None
    public_methods = None
    extra_response_fields = None
    embedded_fields = None
    datasource = None
    versioning = None
    internal_resource = None
    resource_title = None
    service = None
    endpoint_schema = None
    resource_preferences = None

    def __init__(self, endpoint_name, app, service, endpoint_schema=None):
        self.endpoint_name = endpoint_name
        self.service = service
        if not endpoint_schema:
            endpoint_schema = {'schema': self.schema}
            if self.additional_lookup is not None:
                endpoint_schema.update({'additional_lookup': self.additional_lookup})
            if self.extra_response_fields is not None:
                endpoint_schema.update({'extra_response_fields': self.extra_response_fields})
            if self.datasource is not None:
                endpoint_schema.update({'datasource': self.datasource})
            if self.item_methods is not None:
                endpoint_schema.update({'item_methods': self.item_methods})
            if self.resource_methods is not None:
                endpoint_schema.update({'resource_methods': self.resource_methods})
            if self.public_methods is not None:
                endpoint_schema.update({'public_methods': self.public_methods})
            if self.url is not None:
                endpoint_schema.update({'url': self.url})
            if self.item_url is not None:
                endpoint_schema.update({'item_url': self.item_url})
            if self.embedded_fields is not None:
                endpoint_schema.update({'embedded_fields': self.embedded_fields})
            if self.versioning is not None:
                endpoint_schema.update({'versioning': self.versioning})
            if self.internal_resource is not None:
                endpoint_schema.update({'internal_resource': self.internal_resource})
            if self.resource_title is not None:
                endpoint_schema.update({'resource_title': self.resource_title})
        self.endpoint_schema = endpoint_schema

        on_fetched_resource = getattr(app, 'on_fetched_resource_%s' % self.endpoint_name)
        on_fetched_resource -= service.on_fetched
        on_fetched_resource += service.on_fetched
        on_insert_event = getattr(app, 'on_insert_%s' % self.endpoint_name)
        on_insert_event -= service.on_create
        on_insert_event += service.on_create
        on_inserted_event = getattr(app, 'on_inserted_%s' % self.endpoint_name)
        on_inserted_event -= service.on_created
        on_inserted_event += service.on_created
        on_update_event = getattr(app, 'on_update_%s' % self.endpoint_name)
        on_update_event -= service.on_update
        on_update_event += service.on_update
        on_updated_event = getattr(app, 'on_updated_%s' % self.endpoint_name)
        on_updated_event -= service.on_updated
        on_updated_event += service.on_updated
        on_delete_event = getattr(app, 'on_delete_item_%s' % self.endpoint_name)
        on_delete_event -= service.on_delete
        on_delete_event += service.on_delete
        on_deleted_event = getattr(app, 'on_deleted_item_%s' % self.endpoint_name)
        on_deleted_event -= service.on_deleted
        on_deleted_event += service.on_deleted

        app.register_resource(self.endpoint_name, endpoint_schema)
        superdesk.resources[self.endpoint_name] = self

    @staticmethod
    def rel(resource, embeddable=True, required=False, type='objectid'):
        return {
            'type': type,
            'required': required,
            'data_relation': {'resource': resource, 'field': '_id', 'embeddable': embeddable}
        }



"""Upload module"""
import logging
import superdesk
from eve.utils import config
from superdesk import SuperdeskError
from .resource import Resource
from .services import BaseService
from flask import url_for, Response, current_app as app, json
from superdesk.media.renditions import generate_renditions, delete_file_on_error
from superdesk.media.media_operations import download_file_from_url, process_file_from_stream, \
    crop_image, decode_metadata, download_file_from_encoded_str


bp = superdesk.Blueprint('upload_raw', __name__)
superdesk.blueprint(bp)
logger = logging.getLogger(__name__)


@bp.route('/upload/<path:media_id>/raw', methods=['GET'])
def get_upload_as_data_uri(media_id):
    media_file = app.media.get(media_id)
    if media_file:
        return Response(media_file.read(), mimetype=media_file.content_type)
    raise SuperdeskError(status_code=404, payload='File not found on media storage.')


def url_for_media(media_id):
    return url_for('upload_raw.get_upload_as_data_uri', media_id=media_id,
                   _external=True, _schema=superdesk.config.URL_PROTOCOL)


def init_app(app):
    endpoint_name = 'upload'
    service = UploadService(endpoint_name, backend=superdesk.get_backend())
    UploadResource(endpoint_name, app=app, service=service)


class UploadResource(Resource):
    schema = {
        'media': {'type': 'file'},
        'CropLeft': {'type': 'integer'},
        'CropRight': {'type': 'integer'},
        'CropTop': {'type': 'integer'},
        'CropBottom': {'type': 'integer'},
        'URL': {'type': 'string'},
        'mime_type': {'type': 'string'},
        'filemeta': {'type': 'dict'}
    }
    extra_response_fields = ['renditions']
    datasource = {
        'projection': {
            'mime_type': 1,
            'filemeta': 1,
            '_created': 1,
            '_updated': 1,
            '_etag': 1,
            'media': 1,
            'renditions': 1,
        }
    }
    item_methods = ['GET', 'DELETE']
    resource_methods = ['GET', 'POST']


class UploadService(BaseService):

    def on_create(self, docs):
        for doc in docs:
            if doc.get('URL') and doc.get('media'):
                message = 'Uploading file by URL and file stream in the same time is not supported.'
                raise SuperdeskError(payload=message)

            content = None
            filename = None
            content_type = None
            if doc.get('media'):
                content = doc['media']
                filename = content.filename
                content_type = content.mimetype
            elif doc.get('URL'):
                content, filename, content_type = self.download_file(doc)

            self.store_file(doc, content, filename, content_type)

    def store_file(self, doc, content, filename, content_type):
        res = process_file_from_stream(content, filename=filename, content_type=content_type)
        file_name, content_type, metadata = res

        cropping_data = self.get_cropping_data(doc)
        _, out = crop_image(content, filename, cropping_data)
        metadata['length'] = json.dumps(len(out.getvalue()))

        try:
            logger.debug('Going to save media file with %s ' % file_name)
            out.seek(0)
            id = app.media.put(out, filename=file_name, content_type=content_type, metadata=metadata)
            doc['media'] = id
            doc['mime_type'] = content_type
            doc['filemeta'] = decode_metadata(metadata)
            inserted = [doc['media']]
            file_type = content_type.split('/')[0]

            rendition_spec = config.RENDITIONS['avatar']
            renditions = generate_renditions(out, doc['media'], inserted, file_type,
                                             content_type, rendition_spec, url_for_media)
            doc['renditions'] = renditions
        except Exception as io:
            logger.exception(io)
            for file_id in inserted:
                delete_file_on_error(doc, file_id)
            raise SuperdeskError(message='Generating renditions failed')

    def get_cropping_data(self, doc):
        if all([doc.get('CropTop', None) is not None, doc.get('CropLeft', None) is not None,
                doc.get('CropRight', None) is not None, doc.get('CropBottom', None) is not None]):
            cropping_data = (doc['CropLeft'], doc['CropTop'], doc['CropRight'], doc['CropBottom'])
            return cropping_data
        return None

    def download_file(self, doc):
        url = doc.get('URL')
        if not url:
            return
        if url.startswith('data'):
            return download_file_from_encoded_str(url)
        else:
            return download_file_from_url(url)



from superdesk.resource import Resource
from bson.objectid import ObjectId
from superdesk.services import BaseService
import superdesk

desks_schema = {
    'name': {
        'type': 'string',
        'unique': True,
        'required': True,
    },
    'description': {
        'type': 'string'
    },
    'members': {
        'type': 'list',
        'schema': {
            'type': 'dict',
            'schema': {
                'user': Resource.rel('users', True)
            }
        }
    },
    'incoming_stage': Resource.rel('stages', True)
}


def init_app(app):
    endpoint_name = 'desks'
    service = DesksService(endpoint_name, backend=superdesk.get_backend())
    DesksResource(endpoint_name, app=app, service=service)
    endpoint_name = 'user_desks'
    service = UserDesksService(endpoint_name, backend=superdesk.get_backend())
    UserDesksResource(endpoint_name, app=app, service=service)


class DesksResource(Resource):
    schema = desks_schema
    datasource = {'default_sort': [('created', -1)]}


class DesksService(BaseService):

    def create(self, docs, **kwargs):
        for doc in docs:
            if not doc.get('incoming_stage', None):
                stage = {'name': '<to be defined>', 'default_incoming': True}
                superdesk.get_resource_service('stages').post([stage])
                doc['incoming_stage'] = stage.get('_id')
                super().create([doc], **kwargs)
                superdesk.get_resource_service('stages').patch(doc['incoming_stage'], {'desk': doc['_id']})
        return [doc['_id'] for doc in docs]


class UserDesksResource(Resource):
    url = 'users/<regex("[a-f0-9]{24}"):user_id>/desks'
    schema = desks_schema
    datasource = {'source': 'desks'}
    resource_methods = ['GET']


class UserDesksService(BaseService):

    def get(self, req, lookup):
        if lookup.get('user_id'):
            lookup["members.user"] = ObjectId(lookup['user_id'])
            del lookup['user_id']
        return super().get(req, lookup)




from datetime import datetime
from superdesk.tests import TestCase
from superdesk.utc import get_date


class UTCTestCase(TestCase):

    def test_get_date(self):
        self.assertIsInstance(get_date('2012-12-12'), datetime)
        self.assertIsInstance(get_date(datetime.now()), datetime)
        self.assertIsNone(get_date(None))



import os
import unittest
from superdesk.etree import etree
import test

from superdesk.io import newsml_2_0


class ItemTest(unittest.TestCase):

    def setUpFixture(self, filename):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.join(dirname, 'fixtures', filename)
        with open(fixture) as f:
            self.tree = etree.fromstring(f.read().encode('utf-8'))
        parser = newsml_2_0.Parser()
        self.item = parser.parse_message(self.tree)[0]


class TextParserTest(ItemTest):
    def setUp(self):
        self.setUpFixture('text.xml')

    def test_instance(self):
        self.assertTrue(self.item)

    def test_parse_id(self):
        self.assertEquals("tag:reuters.com,0000:newsml_L4N0BT5PJ", self.item.get('guid'))
        self.assertEquals('263518268', self.item.get('version'))
        self.assertEquals(self.item.get('guid'), self.item.get('uri'))

    def test_parse_item_meta(self):
        self.assertEquals("text", self.item.get('type'))
        self.assertEquals("reuters.com", self.item.get('provider'))
        self.assertEquals("2013-03-01T15:09:04", self.item.get('versioncreated').isoformat())
        self.assertEquals("2013-03-01T15:09:04", self.item.get('firstcreated').isoformat())

    def test_parse_content_meta(self):
        self.assertEquals('3', self.item.get('urgency'))
        self.assertEquals("SOCCER-ENGLAND/CHELSEA-BENITEZ", self.item["slugline"])
        self.assertEquals("Soccer-Smiling Benitez pleads for support after midweek outburst", self.item["headline"])
        self.assertEquals("Reuters", self.item["creditline"])
        self.assertEquals("SOCCER-ENGLAND/CHELSEA-BENITEZ:Soccer-Smiling Benitez pleads for support after midweek outburst", self.item.get('description_text'))  # noqa

    def test_parse_rights_info(self):
        self.assertEquals("Thomson Reuters", self.item.get('copyrightholder'))
        self.assertEquals("(c) Copyright Thomson Reuters 2013. Click For Restrictions - http://about.reuters.com/fulllegal.asp", self.item.get('copyrightnotice'))  # noqa

    def test_content_set(self):
        self.assertEquals("<p>By Toby Davis</p>", self.item.get('body_html'))

    def test_language(self):
        self.assertEquals('en', self.item.get('language'))

    def test_subject(self):
        self.assertEquals(2, len(self.item.get('subject')))
        self.assertIn({'code': '15054000', 'name': 'soccer'}, self.item.get('subject'))

    def test_pubstatus(self):
        self.assertEquals('usable', self.item.get('pubstatus'))


class PictureParserTest(ItemTest):
    def setUp(self):
        self.setUpFixture('picture.xml')

    def test_type(self):
        self.assertEquals('picture', self.item.get('type'))

    def test_content_set(self):
        self.assertEquals(3, len(self.item.get('renditions')))

        remote = self.item.get('renditions').get('baseImage')
        self.assertTrue(remote)
        self.assertEquals("tag:reuters.com,0000:binary_GM1E9341HD701-BASEIMAGE", remote.get('residRef'))
        self.assertEquals(772617, remote.get('sizeinbytes'))
        self.assertEquals("image/jpeg", remote.get('mimetype'))
        self.assertEquals("http://content.reuters.com/auth-server/content/tag:reuters.com,0000:newsml_GM1E9341HD701:360624134/tag:reuters.com,0000:binary_GM1E9341HD701-BASEIMAGE", remote.get('href'))  # noqa

    def test_byline(self):
        self.assertEquals('MARKO DJURICA', self.item.get('byline'))

    def test_place(self):
        self.assertEquals(2, len(self.item.get('place')))
        self.assertIn({'name': 'NAIROBI'}, self.item['place'])
        self.assertIn({'name': 'Kenya'}, self.item['place'])


class SNEPParserTest(ItemTest):
    def setUp(self):
        self.setUpFixture('snep.xml')

    def test_content_set(self):
        self.assertEquals(2, len(self.item.get('groups')))

        group = self.item.get('groups')[0]
        self.assertTrue(group)
        self.assertEquals("root", group.get('id'))
        self.assertEquals("grpRole:SNEP", group.get('role'))
        self.assertEquals(1, len(group.get('refs')))
        self.assertEquals("main", group.get('refs')[0].get('idRef'))

        group = self.item.get('groups')[1]
        self.assertEquals(10, len(group.get('refs')))
        self.assertEquals("main", group.get('id'))

        ref = group.get('refs')[0]
        self.assertTrue(ref)
        self.assertEquals("tag:reuters.com,0000:newsml_BRE9220HA", ref.get('residRef'))
        self.assertEquals("application/vnd.iptc.g2.packageitem+xml", ref.get('contentType'))
        self.assertEquals("icls:composite", ref.get('itemClass'))
        self.assertEquals("reuters.com", ref.get('provider'))
        self.assertEquals("At least 15 killed on Kenya coast on election day", ref.get('headline'))

if __name__ == '__main__':
    test.drop_db()
    unittest.main()



from flask import abort, current_app as app
from superdesk.media.media_operations import process_file_from_stream, decode_metadata
from superdesk.media.renditions import generate_renditions, delete_file_on_error
from superdesk.resource import Resource
from superdesk.upload import url_for_media
from superdesk.utc import utcnow
from eve.utils import config
from .common import base_schema, item_url, update_dates_for, generate_guid, GUID_TAG, ARCHIVE_MEDIA, set_user
from .common import on_create_media_archive, on_update_media_archive, on_delete_media_archive
from superdesk.activity import add_activity
import logging
from superdesk.services import BaseService

logger = logging.getLogger(__name__)


class ArchiveMediaResource(Resource):
    endpoint_name = ARCHIVE_MEDIA
    schema = {
        'media': {
            'type': 'file',
            'required': True
        },
        'upload_id': {'type': 'string'},
        'headline': base_schema['headline'],
        'byline': base_schema['byline'],
        'description_text': base_schema['description_text'],
        'creator': base_schema['creator']
    }
    datasource = {'source': 'archive'}
    resource_methods = ['POST']
    item_methods = ['PATCH', 'GET', 'DELETE']
    item_url = item_url


class ArchiveMediaService(BaseService):

    type_av = {'image': 'picture', 'audio': 'audio', 'video': 'video'}

    def on_update(self, updates, original):
        on_update_media_archive()

    def on_delete(self, doc):
        on_delete_media_archive()

    def on_create(self, docs):
        ''' Create corresponding item on file upload '''
        for doc in docs:
            file, content_type, metadata = self.get_file_from_document(doc)
            inserted = [doc['media']]
            file_type = content_type.split('/')[0]

            try:
                update_dates_for(doc)
                doc['guid'] = generate_guid(type=GUID_TAG)
                doc.setdefault('_id', doc['guid'])
                doc['type'] = self.type_av.get(file_type)
                doc['version'] = 1
                doc['versioncreated'] = utcnow()
                rendition_spec = config.RENDITIONS['picture']
                renditions = generate_renditions(file, doc['media'], inserted, file_type,
                                                 content_type, rendition_spec, url_for_media)
                doc['renditions'] = renditions
                doc['mimetype'] = content_type
                doc['filemeta'] = metadata
                if not doc.get('_import', None):
                    doc['creator'] = set_user(doc)

                add_activity('uploaded media {{ name }}',
                             name=doc.get('headline', doc.get('mimetype')),
                             renditions=doc.get('renditions'))

            except Exception as io:
                logger.exception(io)
                for file_id in inserted:
                    delete_file_on_error(doc, file_id)
                abort(500)
        on_create_media_archive()

    def get_file_from_document(self, doc):
        file = doc.get('media_fetched')
        if file:
            del doc['media_fetched']
        else:
            content = doc['media']
            res = process_file_from_stream(content, filename=content.filename, content_type=content.mimetype)
            file_name, content_type, metadata = res
            logger.debug('Going to save media file with %s ' % file_name)
            content.seek(0)
            id = app.media.put(content, filename=file_name, content_type=content_type, metadata=metadata)
            doc['media'] = id
            return content, content_type, decode_metadata(metadata)
        return file, file.content_type, file.metadata



import logging
from flask import current_app as app
import superdesk
from superdesk.utc import utcnow
from superdesk.utils import get_hash, is_hashed


logger = logging.getLogger(__name__)


class CreateUserCommand(superdesk.Command):
    """Create a user with given username, password and email.
    If user with given username exists, reset password.
    """

    option_list = (
        superdesk.Option('--username', '-u', dest='username', required=True),
        superdesk.Option('--password', '-p', dest='password', required=True),
        superdesk.Option('--email', '-e', dest='email', required=True),
        superdesk.Option('--admin', '-a', dest='admin', required=False, default='false'),
    )

    def run(self, username, password, email, admin='false'):

        # force type conversion to boolean
        user_type = 'administrator' if admin.lower() == 'true' else 'user'

        userdata = {
            'username': username,
            'password': password,
            'email': email,
            'user_type': user_type,
            app.config['LAST_UPDATED']: utcnow(),
        }

        with app.test_request_context('/users', method='POST'):
            if userdata.get('password', None) and not is_hashed(userdata.get('password')):
                userdata['password'] = get_hash(userdata.get('password'),
                                                app.config.get('BCRYPT_GENSALT_WORK_FACTOR', 12))

            user = app.data.find_one('users', username=userdata.get('username'), req=None)

            if user:
                logger.info('updating user %s' % (userdata))
                app.data.update('users', user.get('_id'), userdata)
                return userdata
            else:
                logger.info('creating user %s' % (userdata))
                userdata[app.config['DATE_CREATED']] = userdata[app.config['LAST_UPDATED']]
                app.data.insert('users', [userdata])

            logger.info('user saved %s' % (userdata))
            return userdata


class HashUserPasswordsCommand(superdesk.Command):
    def run(self):
        users = superdesk.get_resource_service('auth_users').get(req=None, lookup={})
        for user in users:
            pwd = user.get('password')
            if not is_hashed(pwd):
                updates = {}
                hashed = get_hash(user['password'], app.config.get('BCRYPT_GENSALT_WORK_FACTOR', 12))
                user_id = user.get('_id')
                updates['password'] = hashed
                superdesk.get_resource_service('users').patch(user_id, updates=updates)


superdesk.command('users:create', CreateUserCommand())
superdesk.command('users:hash_passwords', HashUserPasswordsCommand())



from superdesk.resource import Resource
from superdesk.notification import push_notification
from superdesk.services import BaseService
import superdesk


def init_app(app):
    endpoint_name = 'coverages'
    service = CoverageService(endpoint_name, backend=superdesk.get_backend())
    CoverageResource(endpoint_name, app=app, service=service)


class CoverageResource(Resource):
    schema = {
        'headline': {'type': 'string'},
        'coverage_type': {
            'type': 'string',
            'allowed': ['story', 'photo', 'video', 'graphics', 'live-blogging'],
            'default': 'story',
            'required': True
        },
        'ed_note': {'type': 'string'},
        'scheduled': {'type': 'datetime'},
        'delivery': {'type': 'string'},
        'assigned_user': Resource.rel('users', True),
        'assigned_desk': Resource.rel('desks', True),
        'planning_item': Resource.rel('planning', True, type='string'),
    }

    datasource = {'default_sort': [('_created', -1)]}


class CoverageService(BaseService):

    def on_created(self, docs):
        push_notification('coverages', created=1)

    def on_updated(self, updates, original):
        push_notification('coverages', updated=1)

    def on_deleted(self, doc):
        push_notification('coverages', deleted=1)



from superdesk import app_models
from .base_model import BaseModel


def register_model(model):
    '''
    Register a model - to be used in bootstrap.
    @param model: object
        The model object.
    '''
    assert isinstance(model, BaseModel), 'Invalid model %s' % model
    app_models[model.name()] = model


def get_model(name):
    '''
    Return an instance of the model identified by given name.
    '''
    if isinstance(name, BaseModel) or (type(name) is type and issubclass(name, BaseModel)):
        name = name.name()
    return app_models[name]



from superdesk.resource import Resource


class SesssionsResource(Resource):
    schema = {
        'user': Resource.rel('users', True)
    }
    datasource = {
        'source': 'auth',
        'default_sort': [('_created', -1)],
        'filter': {'$where': '(ISODate() - this._created) / 3600000 <= 12'}  # last 12h
    }
    resource_methods = ['GET']
    item_methods = []
    embedded_fields = ['user']



import json
import os

from superdesk import get_resource_service
import superdesk


def apply_placeholders(placeholders, text):
    if not placeholders or not text:
        return text
    for tag, value in placeholders.items():
        text = text.replace(tag, value)
    return text


class AppPrepopulateCommand(superdesk.Command):
    def run(self):
        placeholders = {}
        file = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'app_prepopulate_data.json')
        with open(file, 'rt') as app_prepopulation:
            json_data = json.loads(app_prepopulation.read())
            for item in json_data:
                service = get_resource_service(item.get('resource', None))
                text = json.dumps(item.get('data', None))
                text = apply_placeholders(placeholders, text)
                data = json.loads(text)
                try:
                    item_id = service.post([data])
                    if 'id_name' in item:
                        placeholders[item.get('id_name')] = str(item_id[0])
                except Exception as e:
                    print(e)


superdesk.command('app:prepopulate', AppPrepopulateCommand())



"""Superdesk IO"""

import logging
from superdesk.celery_app import celery


providers = {}
allowed_providers = []
logger = logging.getLogger(__name__)


class IngestService():
    """Base ingest service class."""

    def get_items(self, guid):
        raise LookupError()

    def update(self, provider):
        raise NotImplementedError()


from .commands.remove_expired_content import RemoveExpiredContent
from .commands.update_ingest import UpdateIngest
from .commands.add_provider import AddProvider  # NOQA


def init_app(app):
    from .ingest_provider_model import IngestProviderResource
    from superdesk.services import BaseService
    import superdesk
    endpoint_name = 'ingest_providers'
    service = BaseService(endpoint_name, backend=superdesk.get_backend())
    IngestProviderResource(endpoint_name, app=app, service=service)


def register_provider(type, provider):
    providers[type] = provider
    allowed_providers.append(type)


@celery.task()
def fetch_ingest():
    try:
        RemoveExpiredContent().run()
    except Exception as ex:
        logger.error(ex)
    UpdateIngest().run()



import logging
import superdesk
from flask import current_app as app, render_template
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.utc import utcnow
from superdesk.utils import get_random_string, get_hash
from settings import RESET_PASSWORD_TOKEN_TIME_TO_LIVE as token_ttl, ADMINS
from superdesk.emails import send_email


logger = logging.getLogger(__name__)


reset_schema = {
    'email': {'type': 'email'},
    'token': {'type': 'string'},
    'password': {'type': 'string'},
    'user': Resource.rel('users', True)
}


class ActiveTokensResource(Resource):
    internal_resource = True
    schema = reset_schema
    where_clause = '(ISODate() - this._created) / 3600000 <= %s' % token_ttl
    datasource = {
        'source': 'reset_user_password',
        'default_sort': [('_created', -1)],
        'filter': {'$where': where_clause}
    }
    resource_methods = []
    item_methods = []


class ResetPasswordResource(Resource):
    schema = reset_schema
    public_methods = ['POST']
    resource_methods = ['POST']
    item_methods = []


class ResetPasswordService(BaseService):

    def create(self, docs, **kwargs):
        for doc in docs:
            email = doc.get('email')
            key = doc.get('token')
            password = doc.get('password')

            if key and password:
                return self.reset_password(doc)
            if email:
                return self.initialize_reset_password(doc, email)
            raise superdesk.SuperdeskError(payload='Invalid request.')

    def initialize_reset_password(self, doc, email):
        user = superdesk.get_resource_service('users').find_one(req=None, email=email)
        if not user:
            logger.warning('User password reset triggered with invalid email: %s' % email)
            raise superdesk.SuperdeskError(status_code=201, message='Created')
        doc[app.config['DATE_CREATED']] = utcnow()
        doc[app.config['LAST_UPDATED']] = utcnow()
        doc['user'] = user['_id']
        doc['token'] = get_random_string()
        ids = super().create([doc])
        self.send_reset_password_email(doc)
        self.remove_private_data(doc)
        return ids

    def reset_password(self, doc):
        key = doc.get('token')
        password = doc.get('password')

        reset_request = superdesk.get_resource_service('active_tokens').find_one(req=None, token=key)
        if not reset_request:
            raise superdesk.SuperdeskError(payload='Invalid token received: %s' % key)

        user_id = reset_request['user']
        user = superdesk.get_resource_service('users').find_one(req=None, _id=user_id)
        if not user:
            raise superdesk.SuperdeskError(payload='Invalid user.')

        self.update_user_password(user_id, password)
        self.remove_all_tokens_for_email(reset_request['email'])
        self.remove_private_data(doc)
        return [reset_request['_id']]

    def remove_all_tokens_for_email(self, email):
        super().delete(lookup={'email': email})

    def update_user_password(self, user_id, password):
        updates = {}
        updates['password'] = get_hash(password, app.config.get('BCRYPT_GENSALT_WORK_FACTOR', 12))
        updates[app.config['LAST_UPDATED']] = utcnow()
        superdesk.get_resource_service('users').patch(user_id, updates=updates)

    def remove_private_data(self, doc):
        self.remove_field_from(doc, 'password')
        self.remove_field_from(doc, 'token')
        self.remove_field_from(doc, 'user')

    def remove_field_from(self, doc, field_name):
        if doc and doc.get(field_name):
            del doc[field_name]

    def send_reset_password_email(self, doc):
        send_email.delay(subject='Reset password',
                         sender=ADMINS[0],
                         recipients=[doc['email']],
                         text_body=render_template("reset_password.txt", user=doc, expires=token_ttl),
                         html_body=render_template("reset_password.html", user=doc, expires=token_ttl))



from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk import get_backend
from eve.validation import ValidationError
import superdesk


_preferences_key = 'preferences'


def init_app(app):
    endpoint_name = 'preferences'
    service = PreferencesService(endpoint_name, backend=get_backend())
    PreferencesResource(endpoint_name, app=app, service=service)


class PreferencesResource(Resource):
    datasource = {'source': 'users', 'projection': {'preferences': 1}}
    schema = {
        _preferences_key: {'type': 'dict', 'required': True}
    }
    resource_methods = []
    item_methods = ['GET', 'PATCH']


class PreferencesService(BaseService):

    def on_update(self, updates, original):
        prefs = updates.get(_preferences_key, {})
        for k in ((k for k, v in prefs.items() if k not in superdesk.available_preferences)):
            raise ValidationError('Invalid preference: %s' % k)

        for k, v in prefs.items():
            new_value = dict(superdesk.available_preferences[k])
            new_value.update(v)
            prefs[k] = new_value

    def find_one(self, req, **lookup):
        doc = super().find_one(req, **lookup)
        self.enhance_document_with_default_prefs(doc)
        return doc

    def get(self, req, lookup):
        docs = super().get(req, lookup)
        for doc in docs:
            self.enhance_document_with_default_prefs(doc)
        return docs

    def enhance_document_with_default_prefs(self, doc):
        orig_prefs = doc.get(_preferences_key, {})
        available = dict(superdesk.available_preferences)
        available.update(orig_prefs)
        doc[_preferences_key] = available

    def get_user_preference(self, user_id, preference_name):
        doc = self.find_one(req=None, _id=user_id)
        prefs = doc.get(_preferences_key, {}).get(preference_name, {})
        return prefs

    def email_notification_is_enabled(self, user_id):
        send_email = self.get_user_preference(user_id, 'email:notification')
        return send_email and send_email.get('enabled', False)



from superdesk.notification import push_notification
from superdesk.resource import Resource
from apps.archive.common import on_create_item
from superdesk.services import BaseService
import superdesk


def init_app(app):
    endpoint_name = 'planning'
    service = PlanningService(endpoint_name, backend=superdesk.get_backend())
    PlanningResource(endpoint_name, app=app, service=service)


class PlanningResource(Resource):
    schema = {
        'guid': {
            'type': 'string',
            'unique': True
        },
        'language': {
            'type': 'string'
        },
        'headline': {
            'type': 'string'
        },
        'slugline': {
            'type': 'string'
        },
        'description_text': {
            'type': 'string',
            'nullable': True
        },
        'firstcreated': {
            'type': 'datetime'
        },
        'urgency': {
            'type': 'integer'
        },
        'desk': Resource.rel('desks', True)
    }
    item_url = 'regex("[\w,.:-]+")'
    datasource = {'search_backend': 'elastic'}
    resource_methods = ['GET', 'POST']


class PlanningService(BaseService):

    def on_create(self, docs):
        on_create_item(docs)

    def on_created(self, docs):
        push_notification('planning', created=1)

    def on_updated(self, updates, original):
        push_notification('planning', updated=1)

    def on_deleted(self, doc):
        push_notification('planning', deleted=1)



from flask import g, request
import superdesk
from superdesk.resource import Resource
from superdesk.notification import push_notification
from superdesk.services import BaseService
from .user_mentions import get_users, get_users_mentions, notify_mentioned_users

comments_schema = {
    'text': {
        'type': 'string',
        'minlength': 1,
        'maxlength': 500,
        'required': True,
    },
    'item': {'type': 'string'},
    'user': Resource.rel('users', True),
    'mentioned_users': {
        'type': 'dict'
    }
}


class CommentsResource(Resource):
    """Reusable implementation for comments."""
    schema = comments_schema
    resource_methods = ['GET', 'POST', 'DELETE']
    datasource = {'default_sort': [('_created', -1)]}


class CommentsService(BaseService):
    notification_key = 'comments'

    def on_create(self, docs):
        for doc in docs:
            sent_user = doc.get('user', None)
            user = g.user
            if sent_user and sent_user != str(user.get('_id')):
                payload = 'Commenting on behalf of someone else is prohibited.'
                raise superdesk.SuperdeskError(payload=payload)
            doc['user'] = str(user.get('_id'))
            usernames = get_users_mentions(doc.get('text'))
            doc['mentioned_users'] = get_users(usernames)

    def on_created(self, docs):
        for doc in docs:
            push_notification(self.notification_key, item=str(doc.get('item')))

        origin = next((value for (key, value) in request.headers if key == 'Origin'))
        notify_mentioned_users(docs, origin)

    def on_updated(self, updates, original):
        push_notification(self.notification_key, updated=1)

    def on_deleted(self, doc):
        push_notification(self.notification_key, deleted=1)



from flask import request
from superdesk.resource import Resource
from .common import get_user, item_url
from superdesk.services import BaseService
from apps.common.components.utils import get_component
from apps.item_lock.components.item_lock import ItemLock


class ArchiveLockResource(Resource):
    endpoint_name = 'archive_lock'
    url = 'archive/<{0}:item_id>/lock'.format(item_url)
    schema = {'lock_user': {'type': 'string'}}
    datasource = {'source': 'archive'}
    resource_methods = ['GET', 'POST']
    resource_title = endpoint_name


class ArchiveLockService(BaseService):

    def create(self, docs, **kwargs):
        user = get_user(required=True)
        item_id = request.view_args['item_id']
        get_component(ItemLock).lock({'_id': request.view_args['item_id']}, user['_id'], None)
        return [item_id]


class ArchiveUnlockResource(Resource):
    endpoint_name = 'archive_unlock'
    url = 'archive/<{0}:item_id>/unlock'.format(item_url)
    schema = {'lock_user': {'type': 'string'}}
    datasource = {'source': 'archive'}
    resource_methods = ['GET', 'POST']
    resource_title = endpoint_name


class ArchiveUnlockService(BaseService):

    def create(self, docs, **kwargs):
        user = get_user(required=True)
        item_id = request.view_args['item_id']
        get_component(ItemLock).unlock({'_id': item_id}, user['_id'], None)
        return [item_id]



import json
import logging

from eve.utils import ParsedRequest

import superdesk
from .common import base_schema, get_user
from superdesk.resource import Resource, build_custom_hateoas
from superdesk.json_path_tool import json_merge_values, json_copy_values
from superdesk.services import BaseService


logger = logging.getLogger(__name__)


def init_parsed_request(filter):
    parsed_request = ParsedRequest()
    if filter:
        parsed_request.args = {'source': json.dumps(filter)}
    return parsed_request


class ContentViewResource(Resource):
    endpoint_name = 'content_view'
    schema = {
        'name': {
            'type': 'string',
            'required': True,
            'minlength': 1
        },
        'location': {
            'type': 'string',
            'allowed': ['ingest', 'archive'],
            'default': 'archive'
        },
        'description': {
            'type': 'string',
            'minlength': 1
        },
        'desk': Resource.rel('desks', True),
        'user': Resource.rel('users', True),
        'filter': {
            'type': 'dict'
        },
        'hateoas': {
            'self': '/{location}/{_id}'
        }
    }


class ContentViewService(BaseService):

    def check_filter(self, filter, location):
        parsed_request = init_parsed_request(filter)
        payload = None
        try:
            superdesk.get_resource_service(location).get(req=parsed_request, lookup={})
        except Exception as e:
            logger.exception(e)
            payload = 'Fail to validate the filter against %s.' % location
        if payload:
            raise superdesk.SuperdeskError(payload=payload)

    def process_and_validate(self, doc):
        if 'desks' in doc and not doc['desks']:
            del doc['desks']

        if 'filter' in doc and doc['filter']:
            self.check_filter(doc['filter'], doc['location'])

    def on_create(self, docs):
        for doc in docs:
            doc.setdefault('user', get_user(required=True)['_id'])
            self.process_and_validate(doc)

    def on_update(self, updates, original):
        self.process_and_validate(updates)


def merge_query(first, second):
    return {"bool": {"should": [first, second], "minimum_should_match": 2}}


def merge_filter(first, second):
    return {'and': [first, second]}


def apply_additional_query(query, additional_query):
    if not query:
        query = additional_query
    elif additional_query:
        json_merge_values(query, additional_query, ['query', 'filtered', 'query'], merge_query)
        json_merge_values(query, additional_query, ['query', 'filtered', 'filter'], merge_filter)
        json_copy_values(query, additional_query, ['size', 'from', 'sort'])

    return query


class ContentViewItemsResource(Resource):
    endpoint_name = 'content_view_items'
    resource_title = endpoint_name
    url = 'content_view/<regex("[a-zA-Z0-9:\\-\\.]+"):content_view_id>/items'
    schema = base_schema
    resource_methods = ['GET']
    datasource = {'backend': 'custom'}


class ContentViewItemsService(BaseService):

    custom_hateoas = {'self': {'title': 'Archive', 'href': '/{location}/{_id}'}}

    def get(self, req, **lookup):
        content_view_id = lookup['lookup']['content_view_id']
        view_items = superdesk.get_resource_service('content_view').find_one(req=None, _id=content_view_id)
        if not view_items:
            raise superdesk.SuperdeskError(payload='Invalid content view id.')
        additional_query = view_items.get('filter')

        query = None
        if req.args.get('source'):
            query = json.loads(req.args.get('source'))

        query = apply_additional_query(query, additional_query)
        parsed_request = init_parsed_request(query)
        location = view_items.get('location', 'archive')
        docs = superdesk.get_resource_service(location).get(req=parsed_request, lookup={})

        for doc in docs:
            build_custom_hateoas(self.custom_hateoas, doc, location=location)
        return docs



import superdesk
from ..models.item_autosave import ItemAutosaveModel
from apps.common.components.base_component import BaseComponent
from apps.common.models.utils import get_model
from apps.item_lock.models.item import ItemModel


class ItemAutosave(BaseComponent):
    def __init__(self, app):
        self.app = app
        self.app.on_item_locked += self.on_item_locked

    @classmethod
    def name(cls):
        return 'archive_autosave'

    def autosave(self, item_id, updates, user, etag):
        item_model = get_model(ItemModel)
        item = item_model.find_one({'_id': item_id})
        if item is None:
            raise superdesk.SuperdeskError('Invalid item identifier', 404)
        if etag:
            item_model.validate_etag(item, etag)

        lock_user = item.get('lock_user', None)
        if lock_user and str(lock_user) != str(user['_id']):
            raise superdesk.SuperdeskError(payload='The item was locked by another user')

        autosave_model = get_model(ItemAutosaveModel)
        item.update(updates)
        self.app.on_item_autosave(item)
        autosave_item = autosave_model.find_one({'_id': item_id})
        if not autosave_item:
            autosave_model.create([item])
        else:
            id = item['_id']
            del item['_id']
            autosave_model.update({'_id': item_id}, item)
            item['_id'] = id
        self.app.on_item_autosaved(item)
        updates.update(item)
        return updates

    def clear(self, item_id):
        autosave_model = get_model(ItemAutosaveModel)
        return autosave_model.delete({'_id': item_id})

    def on_item_locked(self, item, user):
        lock_user = item.get('lock_user', None)
        if str(lock_user) != str(user):
            self.clear(item['_id'])




import datetime
from ..etree import etree


class Parser():
    """NewsMl xml 1.2 parser"""

    def parse_message(self, tree):
        """Parse NewsMessage."""
        item = {}
        self.root = tree

        item['NewsIdentifier'] = self.parse_elements(tree.find('NewsItem/Identification/NewsIdentifier'))
        item['NewsManagement'] = self.parse_elements(tree.find('NewsItem/NewsManagement'))
        item['NewsLines'] = self.parse_elements(tree.find('NewsItem/NewsComponent/NewsLines'))
        item['Provider'] = self.parse_attributes_as_dictionary(
            tree.find('NewsItem/NewsComponent/AdministrativeMetadata/Provider'))
        item['DescriptiveMetadata'] = self.parse_multivalued_elements(
            tree.find('NewsItem/NewsComponent/DescriptiveMetadata'))
        item['located'] = self.parse_attributes_as_dictionary(
            tree.find('NewsItem/NewsComponent/DescriptiveMetadata/Location'))

        keywords = tree.findall('NewsItem/NewsComponent/DescriptiveMetadata/Property')
        item['keywords'] = self.parse_attribute_values(keywords, 'Keyword')

        subjects = tree.findall('NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/SubjectDetail')
        subjects += tree.findall('NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/SubjectMatter')
        subjects += tree.findall('NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/Subject')

        item['Subjects'] = self.parse_attributes_as_dictionary(subjects)
        item['ContentItem'] = self.parse_attributes_as_dictionary(
            tree.find('NewsItem/NewsComponent/ContentItem'))
        # item['Content'] = etree.tostring(
        #       tree.find('NewsItem/NewsComponent/ContentItem/DataContent/nitf/body/body.content'))
        item['body_html'] = etree.tostring(
            tree.find('NewsItem/NewsComponent/ContentItem/DataContent/nitf/body/body.content'))

        return self.populate_fields(item)

    def parse_elements(self, tree):
        items = {}
        for item in tree:
            if item.text is None:
                # read the attribute for the item
                if item.tag != 'HeadLine':
                    items[item.tag] = item.attrib
            else:
                # read the value for the item
                items[item.tag] = item.text
        return items

    def parse_multivalued_elements(self, tree):
        items = {}
        for item in tree:
            if item.tag not in items:
                items[item.tag] = [item.text]
            else:
                items[item.tag].append(item.text)

        return items

    def parse_attributes_as_dictionary(self, items):
        attributes = [item.attrib for item in items]
        return attributes

    def parse_attribute_values(self, items, attribute):
        attributes = []
        for item in items:
            if item.attrib['FormalName'] == attribute:
                attributes.append(item.attrib['Value'])
        return attributes

    def datetime(self, string):
        return datetime.datetime.strptime(string, '%Y%m%dT%H%M%S+0000')

    def populate_fields(self, item):
        item['guid'] = item['NewsIdentifier']['PublicIdentifier']
        item['provider'] = item['Provider'][0]['FormalName']
        item['type'] = 'text'
        item['urgency'] = item['NewsManagement']['Urgency']['FormalName']
        item['version'] = item['NewsIdentifier']['RevisionId']
        item['versioncreated'] = self.datetime(item['NewsManagement']['ThisRevisionCreated'])
        item['firstcreated'] = self.datetime(item['NewsManagement']['FirstCreated'])
        item['pubstatus'] = item['NewsManagement']['Status']['FormalName']
        item['subject'] = item['Subjects']

        if 'HeadLine' in item['NewsLines']:
            item['headline'] = item['NewsLines']['HeadLine']
        elif item['NewsManagement']['NewsItemType']['FormalName'] == 'Alert':
            item['headline'] = 'Alert'

        return item



from settings import LDAP_SERVER
from superdesk.tests import TestCase
from superdesk import get_resource_service
from .commands import CreateUserCommand


class UsersTestCase(TestCase):

    def test_create_user_command(self):
        if not LDAP_SERVER:
            user = {'username': 'foo', 'password': 'bar', 'email': 'baz'}
            cmd = CreateUserCommand()
            with self.app.test_request_context():
                cmd.run(user['username'], user['password'], user['email'])
                auth_user = get_resource_service('auth').authenticate(user)
                self.assertEquals(auth_user['username'], user['username'])

                cmd.run(user['username'], user['password'], user['email'])
                auth_user2 = get_resource_service('auth').authenticate(user)
                self.assertEquals(auth_user2['username'], user['username'])
                self.assertEquals(auth_user2['_id'], auth_user['_id'])




import os
import logging
import importlib
import jinja2
import eve
import settings
import superdesk
from flask.ext.mail import Mail
from eve.io.mongo import MongoJSONEncoder
from eve.render import send_response
from superdesk.celery_app import init_celery
from eve.auth import TokenAuth
from superdesk.storage.desk_media_storage import SuperdeskGridFSMediaStorage
from superdesk.validator import SuperdeskValidator
from raven.contrib.flask import Sentry


logger = logging.getLogger('superdesk')
sentry = Sentry(register_signal=False, wrap_wsgi=False)


def get_app(config=None):
    """App factory.

    :param config: configuration that can override config from `settings.py`
    :return: a new SuperdeskEve app instance
    """
    if config is None:
        config = {}

    for key in dir(settings):
        if key.isupper():
            config.setdefault(key, getattr(settings, key))

    media_storage = SuperdeskGridFSMediaStorage

    if config['AMAZON_CONTAINER_NAME']:
        from superdesk.storage.amazon.amazon_media_storage import AmazonMediaStorage
        from superdesk.storage.amazon.import_from_amazon import ImportFromAmazonCommand
        media_storage = AmazonMediaStorage
        superdesk.command('import:amazon', ImportFromAmazonCommand())

    config['DOMAIN'] = {}

    app = eve.Eve(
        data=superdesk.SuperdeskDataLayer,
        auth=TokenAuth,
        media=media_storage,
        settings=config,
        json_encoder=MongoJSONEncoder,
        validator=SuperdeskValidator)

    custom_loader = jinja2.ChoiceLoader([
        app.jinja_loader,
        jinja2.FileSystemLoader(['superdesk/templates'])
    ])
    app.jinja_loader = custom_loader

    app.mail = Mail(app)

    @app.errorhandler(superdesk.SuperdeskError)
    def client_error_handler(error):
        """Return json error response.

        :param error: an instance of :attr:`superdesk.SuperdeskError` class
        """
        return send_response(None, (error.to_dict(), None, None, error.status_code))

    @app.errorhandler(500)
    def server_error_handler(error):
        """Log server errors."""
        app.sentry.captureException()
        logger.exception(error)
        return_error = superdesk.SuperdeskError(status_code=500)
        return client_error_handler(return_error)

    init_celery(app)

    for module_name in app.config['INSTALLED_APPS']:
        app_module = importlib.import_module(module_name)
        try:
            app_module.init_app(app)
        except AttributeError:
            pass

    for resource in superdesk.DOMAIN:
        app.register_resource(resource, superdesk.DOMAIN[resource])

    for blueprint in superdesk.BLUEPRINTS:
        prefix = app.api_prefix or None
        app.register_blueprint(blueprint, url_prefix=prefix)

    # we can only put mapping when all resources are registered
    app.data.elastic.put_mapping(app)

    app.sentry = sentry
    sentry.init_app(app)
    superdesk.app = app
    return app

if __name__ == '__main__':

    debug = True
    port = int(os.environ.get('PORT', '5000'))
    host = '0.0.0.0'
    superdesk.logger.setLevel(logging.INFO)
    superdesk.logger.addHandler(logging.StreamHandler())

    app = get_app()
    app.run(host=host, port=port, debug=debug, use_reloader=debug)



from .commands.update_ingest import ingest_items
from .reuters_mock import setup_reuters_mock, teardown_reuters_mock
from .reuters import ReutersIngestService
import superdesk


def setup_providers(context):
    app = context.app
    context.providers = {}
    context.provider_services = {}
    context.ingest_items = ingest_items
    with app.test_request_context():
        app.config['REUTERS_USERNAME'] = 'no_username'
        app.config['REUTERS_PASSWORD'] = 'no_password'
        setup_reuters_mock(context)
        provider = {'name': 'reuters',
                    'type': 'reuters',
                    'config': {'username': app.config['REUTERS_USERNAME'],
                               'password': app.config['REUTERS_PASSWORD']
                               }
                    }

        result = superdesk.get_resource_service('ingest_providers').post([provider])
        context.providers['reuters'] = result[0]
        context.provider_services['reuters'] = ReutersIngestService()


def teardown_providers(context):
    teardown_reuters_mock(context)



from superdesk.resource import Resource
from .common import base_schema, extra_response_fields, item_url, facets
from .common import on_create_item, on_create_media_archive, on_update_media_archive, on_delete_media_archive
from .common import get_user
from flask import current_app as app
from werkzeug.exceptions import NotFound
from superdesk import SuperdeskError, get_resource_service
from superdesk.utc import utcnow
from eve.versioning import resolve_document_version
from superdesk.activity import add_activity
from eve.utils import parse_request
from superdesk.services import BaseService
from apps.common.components.utils import get_component
from apps.item_autosave.components.item_autosave import ItemAutosave
from apps.common.models.base_model import InvalidEtag


def get_subject(doc1, doc2=None):
    for key in ('headline', 'subject', 'slugline'):
        value = doc1.get(key)
        if not value and doc2:
            value = doc2.get(key)
        if value:
            return value


class ArchiveVersionsResource(Resource):
    schema = base_schema
    extra_response_fields = extra_response_fields
    item_url = item_url
    resource_methods = []
    internal_resource = True


class ArchiveVersionsService(BaseService):

    def on_create(self, docs):
        for doc in docs:
            doc['versioncreated'] = utcnow()


class ArchiveResource(Resource):
    schema = {
        'old_version': {
            'type': 'number',
        },
        'last_version': {
            'type': 'number',
        }
    }
    schema.update(base_schema)
    extra_response_fields = extra_response_fields
    item_url = item_url
    datasource = {
        'search_backend': 'elastic',
        'facets': facets,
        'projection': {
            'old_version': 0,
            'last_version': 0
        },
        'default_sort': [('_updated', -1)],
    }
    resource_methods = ['GET', 'POST', 'DELETE']
    versioning = True


class ArchiveService(BaseService):

    def on_create(self, docs):
        on_create_item(docs)

    def on_created(self, docs):
        on_create_media_archive()
        for doc in docs:
            add_activity('added new item {{ type }} about {{ subject }}',
                         type=doc['type'], subject=get_subject(doc))

    def on_update(self, updates, original):
        user = get_user()
        lock_user = original.get('lock_user', None)
        force_unlock = updates.get('force_unlock', False)
        if lock_user and str(lock_user) != str(user['_id']) and not force_unlock:
            raise SuperdeskError(payload='The item was locked by another user')
        updates['versioncreated'] = utcnow()
        updates['version_creator'] = str(user.get('_id'))
        if force_unlock:
            del updates['force_unlock']

    def on_updated(self, updates, original):
        get_component(ItemAutosave).clear(original['_id'])
        on_update_media_archive()

        if '_version' in updates:
            add_activity('created new version {{ version }} for item {{ type }} about {{ subject }}',
                         version=updates['_version'], subject=get_subject(updates, original))

    def on_replace(self, document, original):
        user = get_user()
        lock_user = original.get('lock_user', None)
        force_unlock = document.get('force_unlock', False)
        if lock_user and str(lock_user) != str(user['_id']) and not force_unlock:
            raise SuperdeskError(payload='The item was locked by another user')
        document['versioncreated'] = utcnow()
        document['version_creator'] = str(user.get('_id'))
        if force_unlock:
            del document['force_unlock']

    def on_replaced(self, document, original):
        get_component(ItemAutosave).clear(original['_id'])
        on_update_media_archive()

    def on_delete(self, doc):
        '''Delete associated binary files.'''
        if doc and doc.get('renditions'):
            for _name, ref in doc['renditions'].items():
                try:
                    app.media.delete(ref['media'])
                except (KeyError, NotFound):
                    pass

    def on_deleted(self, doc):
        on_delete_media_archive()
        add_activity('removed item {{ type }} about {{ subject }}',
                     type=doc['type'], subject=get_subject(doc))

    def replace(self, id, document):
        return self.restore_version(id, document) or \
            super().replace(id, document)

    def restore_version(self, id, doc):
        item_id = id
        old_version = int(doc.get('old_version', 0))
        last_version = int(doc.get('last_version', 0))
        if(not all([item_id, old_version, last_version])):
            return None

        old = get_resource_service('archive_versions').find_one(req=None, _id_document=item_id, _version=old_version)
        if old is None:
            raise SuperdeskError(payload='Invalid version %s' % old_version)

        curr = get_resource_service('archive').find_one(req=None, _id=item_id)
        if curr is None:
            raise SuperdeskError(payload='Invalid item id %s' % item_id)

        if curr['_version'] != last_version:
            raise SuperdeskError(payload='Invalid last version %s' % last_version)
        old['_id'] = old['_id_document']
        old['_updated'] = old['versioncreated'] = utcnow()
        del old['_id_document']

        resolve_document_version(old, 'archive', 'PATCH', curr)
        res = super().replace(id=item_id, document=old)
        del doc['old_version']
        del doc['last_version']
        doc.update(old)
        return res


class AutoSaveResource(Resource):
    endpoint_name = 'archive_autosave'
    item_url = item_url
    schema = {
        '_id': {'type': 'string'}
    }
    schema.update(base_schema)
    schema['type'] = {'type': 'string'}
    resource_methods = ['POST']
    item_methods = ['GET', 'PUT', 'PATCH']
    resource_title = endpoint_name


class ArchiveSaveService(BaseService):

    def create(self, docs, **kwargs):
        if not docs:
            raise SuperdeskError('Content is missing', 400)
        req = parse_request(self.datasource)
        try:
            get_component(ItemAutosave).autosave(docs[0]['_id'], docs[0], get_user(required=True), req.if_match)
        except InvalidEtag:
            raise SuperdeskError('Client and server etags don\'t match', 412)
        return [docs[0]['_id']]



from apps.common.models.base_model import Validator, BaseModel


class ItemValidator(Validator):
    def validate(self, doc):
        return True


class ItemModel(BaseModel):
    def __init__(self, data_layer):
        BaseModel.__init__(self, 'archive', data_layer, {}, ItemValidator())

    @classmethod
    def name(cls):
        return 'item_model'



from httmock import urlmatch, HTTMock
from urllib.parse import parse_qs
from os.path import os, basename


@urlmatch(scheme='https', netloc='commerce.reuters.com', path='/rmd/rest/xml/login')
def login_request(url, request):
    return {'status_code': 200,
            'content': '<?xml version="1.0" encoding="UTF-8" standalone="yes"?><authToken>fake_token</authToken>'}


@urlmatch(scheme='http', netloc='rmb.reuters.com', path='/rmd/rest/xml/item')
def item_request(url, request):
    try:
        params = parse_qs(url.query, keep_blank_values=True)
        fixtures = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'fixtures')
        file = os.path.join(fixtures, params['id'][0])
        with open(file, "r") as stored_response:
            content = stored_response.read()
            return {'status_code': 200, 'content': content}
    except Exception:
        return {'status_code': 404}


@urlmatch(scheme='http', netloc='content.reuters.com')
def content_request(url, request):
    try:
        fixtures = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'fixtures')
        file = os.path.join(fixtures, basename(url.path))
        with open(file, 'rb') as stored_response:
            content = stored_response.read()
            return {'status_code': 200, 'content': content}
    except Exception:
        return {'status_code': 404}


def setup_reuters_mock(context):
    context.mock = HTTMock(*[login_request, item_request, content_request])
    context.mock.__enter__()


def teardown_reuters_mock(context):
    if hasattr(context, 'mock'):
        context.mock.__exit__(None, None, None)



"""Reuters Token Provider"""

import ssl
import requests
import superdesk
from datetime import timedelta
from requests.packages.urllib3.poolmanager import PoolManager
from superdesk.etree import etree
from superdesk.utc import utcnow


def is_valid_token(token):
    ttl = timedelta(hours=12)
    return token.get('created') + ttl >= utcnow()


def update_provider_token(provider):
    """Update provider token."""
    token = {}
    token['token'] = fetch_token_from_api(provider)
    token['created'] = utcnow()
    provider['token'] = token
    superdesk.get_resource_service('ingest_providers').patch(provider['_id'], {'token': token})
    return token['token']


def get_token(provider, update=False):
    """Get auth token for given provider instance and save it in db."""
    token = provider.get('token')
    if token and (is_valid_token(token) or not update):
        return token.get('token')
    return update_provider_token(provider) if update else ''


def fetch_token_from_api(provider):
    session = requests.Session()
    session.mount('https://', SSLAdapter())

    url = 'https://commerce.reuters.com/rmd/rest/xml/login'
    payload = {
        'username': provider.get('config', {}).get('username', ''),
        'password': provider.get('config', {}).get('password', ''),
    }

    response = session.get(url, params=payload, stream=False, verify=False)
    # workaround for httmock lib
    # tree = etree.fromstring(response.text)
    tree = etree.fromstring(response.content)
    return tree.text


# workaround for ssl version error
class SSLAdapter(requests.adapters.HTTPAdapter):
    """SSL Adapter set for ssl tls v1."""

    def init_poolmanager(self, connections, maxsize, **kwargs):
        """Init poolmanager to use ssl version v1."""
        self.poolmanager = PoolManager(
            num_pools=connections,
            maxsize=maxsize,
            ssl_version=ssl.PROTOCOL_TLSv1,
            **kwargs
        )



from apps.auth import AuthResource
from .reset_password import ResetPasswordService, ResetPasswordResource, ActiveTokensResource
import superdesk
from .db import DbAuthService
from .commands import CreateUserCommand, HashUserPasswordsCommand  # noqa
from superdesk.services import BaseService


def init_app(app):
    endpoint_name = 'auth'
    service = DbAuthService(endpoint_name, backend=superdesk.get_backend())
    AuthResource(endpoint_name, app=app, service=service)

    endpoint_name = 'reset_user_password'
    service = ResetPasswordService(endpoint_name, backend=superdesk.get_backend())
    ResetPasswordResource(endpoint_name, app=app, service=service)

    endpoint_name = 'active_tokens'
    service = BaseService(endpoint_name, backend=superdesk.get_backend())
    ActiveTokensResource(endpoint_name, app=app, service=service)




import os
import unittest

from .nitf import parse


class TestCase(unittest.TestCase):

    def setUp(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.join(dirname, 'fixtures', 'aap.xml')
        with open(fixture) as f:
            self.nitf = f.read()
            self.item = parse(self.nitf)

    def test_headline(self):
        self.assertEquals(self.item.get('headline'), "The main stories on today's 1900 ABC TV news")

    def test_keywords(self):
        self.assertEquals(self.item.get('keywords'), ['Monitor 1900 ABC News'])

    def test_subjects(self):
        self.assertEquals(len(self.item.get('subject')), 2)
        self.assertIn({'name': 'Justice'}, self.item.get('subject'))
        self.assertIn({'code': '02003000', 'name': 'Police'}, self.item.get('subject'))

    def test_guid(self):
        self.assertEquals(self.item.get('guid'), 'AAP.115314987.5417374')
        self.assertEquals(self.item.get('guid'), self.item.get('uri'))

    def test_type(self):
        self.assertEquals(self.item.get('type'), 'text')

    def test_urgency(self):
        self.assertEquals(self.item.get('urgency'), '5')

    def test_copyright(self):
        self.assertEquals(self.item.get('copyrightholder'), 'Australian Associated Press')

    def test_dates(self):
        self.assertEquals(self.item.get('firstcreated').isoformat(), '2013-10-20T19:27:51')
        self.assertEquals(self.item.get('versioncreated').isoformat(), '2013-10-20T19:27:51')

    def test_content(self):
        text = "<p>   1A) More extreme weather forecast over the next few days the <br />fire situation is likely"
        self.assertIn(text, self.item.get('body_html'))

    def test_pubstatus(self):
        self.assertEquals('usable', self.item.get('pubstatus'))

if __name__ == '__main__':
    unittest.main()




import os
import arrow
import magic
import hashlib
import logging
import requests
import superdesk
from io import BytesIO
from PIL import Image
from flask import json
from .image import get_meta
from .video import get_meta as video_meta
import base64

logger = logging.getLogger(__name__)


def hash_file(afile, hasher, blocksize=65536):
    buf = afile.read(blocksize)
    while len(buf) > 0:
        hasher.update(buf)
        buf = afile.read(blocksize)
    return hasher.hexdigest()


def get_file_name(file):
    return hash_file(file, hashlib.sha256())


def download_file_from_url(url):
    rv = requests.get(url)
    if rv.status_code not in (200, 201):
        payload = 'Failed to retrieve file from URL: %s' % url
        raise superdesk.SuperdeskError(payload=payload)

    mime = magic.from_buffer(rv.content, mime=True).decode('UTF-8')
    ext = mime.split('/')[1]
    name = 'stub.' + ext
    return BytesIO(rv.content), name, mime


def download_file_from_encoded_str(encoded_str):
    content = encoded_str.split(';base64,')
    mime = content[0].split(':')[1]
    ext = content[0].split('/')[1]
    name = 'web_capture.' + ext
    content = base64.b64decode(content[1])
    return BytesIO(content), name, mime


def process_file_from_stream(content, filename=None, content_type=None):
    content_type = content_type or content.content_type
    content = BytesIO(content.read())

    if 'application/' in content_type:
        content_type = magic.from_buffer(content.getvalue(), mime=True).decode('UTF-8')
        content.seek(0)

    file_type, ext = content_type.split('/')
    metadata = process_file(content, file_type)
    file_name = get_file_name(content)
    content.seek(0)
    metadata = encode_metadata(metadata)
    metadata.update({'length': json.dumps(len(content.getvalue()))})
    return file_name, content_type, metadata


def encode_metadata(metadata):
    return dict((k.lower(), json.dumps(v)) for k, v in metadata.items())


def decode_metadata(metadata):
    return dict((k.lower(), decode_val(v)) for k, v in metadata.items())


def decode_val(string_val):
    """Format dates that elastic will try to convert automatically."""
    val = json.loads(string_val)
    try:
        arrow.get(val, 'YYYY-MM-DD')  # test if it will get matched by elastic
        return str(arrow.get(val))
    except (Exception):
        return val


def process_file(content, type):
    if type == 'image':
        return process_image(content, type)
    if type in ('audio', 'video'):
        return process_video(content, type)
    return {}


def process_video(content, type):
    content.seek(0)
    meta = video_meta(content)
    content.seek(0)
    return meta


def process_image(content, type):
    content.seek(0)
    meta = get_meta(content)
    content.seek(0)
    return meta


def crop_image(content, file_name, cropping_data):
    if cropping_data:
        file_ext = os.path.splitext(file_name)[1][1:]
        if file_ext in ('JPG', 'jpg'):
            file_ext = 'jpeg'
        logger.debug('Opened image from stream, going to crop it s')
        content.seek(0)
        img = Image.open(content)
        cropped = img.crop(cropping_data)
        logger.debug('Cropped image from stream, going to save it')
        try:
            out = BytesIO()
            cropped.save(out, file_ext)
            out.seek(0)
            return (True, out)
        except Exception as io:
            logger.exception(io)
    return (False, content)



from superdesk.utc import utcnow
from datetime import datetime
from settings import SERVER_DOMAIN
from uuid import uuid4
from superdesk import SuperdeskError
from superdesk.notification import push_notification
import flask
import superdesk
from superdesk.resource import Resource


GUID_TAG = 'tag'
GUID_NEWSML = 'newsml'
ARCHIVE_MEDIA = 'archive_media'


class InvalidFileType(SuperdeskError):
    """Exception raised when receiving a file type that is not supported."""

    def __init__(self, type=None):
        super().__init__('Invalid file type %s' % type, payload={})


def on_create_item(docs):
    """Make sure item has basic fields populated."""
    for doc in docs:
        update_dates_for(doc)
        doc['original_creator'] = set_user(doc)

        if not doc.get('guid'):
            doc['guid'] = generate_guid(type=GUID_NEWSML)

        doc.setdefault('_id', doc['guid'])


def update_dates_for(doc):
    for item in ['firstcreated', 'versioncreated']:
        doc.setdefault(item, utcnow())


def generate_guid(**hints):
    '''Generate a GUID based on given hints'''
    newsml_guid_format = 'urn:newsml:%(domain)s:%(timestamp)s:%(identifier)s'
    tag_guid_format = 'tag:%(domain)s:%(year)d:%(identifier)s'

    if not hints.get('id'):
        hints['id'] = str(uuid4())

    t = datetime.today()

    if hints['type'].lower() == GUID_TAG:
        return tag_guid_format % {'domain': SERVER_DOMAIN, 'year': t.year, 'identifier': hints['id']}
    elif hints['type'].lower() == GUID_NEWSML:
        return newsml_guid_format % {'domain': SERVER_DOMAIN, 'timestamp': t.isoformat(), 'identifier': hints['id']}
    return None


def get_user(required=False):
    user = flask.g.get('user', {})
    if '_id' not in user and required:
        raise superdesk.SuperdeskError(payload='Invalid user.')
    return user


def set_user(doc):
    usr = get_user()
    user = str(usr.get('_id', ''))
    sent_user = doc.get('user', None)
    if sent_user and user and sent_user != user:
        raise superdesk.SuperdeskError()
    doc['user'] = user
    return user


base_schema = {
    'guid': {
        'type': 'string',
        'unique': True
    },
    'provider': {
        'type': 'string'
    },
    'type': {
        'type': 'string',
        'required': True,
        'allowed': ['text', 'audio', 'video', 'picture', 'graphic', 'composite'],
        'default': 'text'
    },
    'mimetype': {
        'type': 'string'
    },
    'version': {
        'type': 'string'
    },
    'versioncreated': {
        'type': 'datetime'
    },
    'pubstatus': {
        'type': 'string'
    },
    'copyrightholder': {
        'type': 'string'
    },
    'copyrightnotice': {
        'type': 'string'
    },
    'usageterms': {
        'type': 'string'
    },
    'language': {
        'type': 'string'
    },
    'place': {
        'type': 'list'
    },
    'subject': {
        'type': 'list'
    },
    'byline': {
        'type': 'string'
    },
    'headline': {
        'type': 'string'
    },
    'located': {
        'type': 'string'
    },
    'renditions': {
        'type': 'dict'
    },
    'slugline': {
        'type': 'string'
    },
    'creditline': {
        'type': 'string'
    },
    'description_text': {
        'type': 'string',
        'nullable': True
    },
    'firstcreated': {
        'type': 'datetime'
    },
    'filemeta': {
        'type': 'dict'
    },
    'ingest_provider': {
        'type': 'string'
    },
    'urgency': {
        'type': 'integer'
    },
    'groups': {
        'type': 'list'
    },
    'keywords': {
        'type': 'list'
    },
    'body_html': {
        'type': 'string'
    },
    'creator': {
        'type': 'dict',
        'schema': {
            'user': Resource.rel('users', True)
        }
    },
    'media_file': {
        'type': 'string'
    },
    'contents': {
        'type': 'list'
    },
    'media': {
        'type': 'media'
    },
    'task_id': {
        'type': 'string'
    },
    'lock_user': {
        'type': 'objectid',
        'data_relation': {'resource': 'users', 'field': '_id', 'embeddable': True}
    },
    'lock_time': {
        'type': 'datetime'
    },
}

item_url = 'regex("[\w,.:_-]+")'

extra_response_fields = ['guid', 'headline', 'firstcreated', 'versioncreated', 'archived']

facets = {
    'type': {'terms': {'field': 'type'}},
    'provider': {'terms': {'field': 'provider'}},
    'urgency': {'terms': {'field': 'urgency'}},
    'subject': {'terms': {'field': 'subject.name'}},
    'place': {'terms': {'field': 'place.name'}},
    'versioncreated': {'date_histogram': {'field': 'versioncreated', 'interval': 'hour'}},
}


def on_create_media_archive():
    push_notification('media_archive', created=1)


def on_update_media_archive():
    push_notification('media_archive', updated=1)


def on_delete_media_archive():
    push_notification('media_archive', deleted=1)




from superdesk.resource import Resource
from superdesk.io import allowed_providers
DAYS_TO_KEEP = 2


class IngestProviderResource(Resource):
    schema = {
        'name': {
            'type': 'string',
            'required': True
        },
        'type': {
            'type': 'string',
            'required': True,
            'allowed': allowed_providers
        },
        'days_to_keep': {
            'type': 'integer',
            'required': True,
            'default': DAYS_TO_KEEP
        },
        'config': {
            'type': 'dict'
        },
        'ingested_count': {
            'type': 'integer'
        },
        'accepted_count': {
            'type': 'integer'
        },
        'token': {
            'type': 'dict'
        }
    }



from superdesk.activity import add_activity
from superdesk.services import BaseService
from superdesk.utils import is_hashed, get_hash
from superdesk import get_resource_service
from flask import current_app as app, render_template
from settings import ADMINS
from superdesk.emails import send_email


import logging


logger = logging.getLogger(__name__)


def get_display_name(user):
    if user.get('first_name') or user.get('last_name'):
        display_name = '%s %s' % (user.get('first_name'), user.get('last_name'))
        return display_name.strip()
    else:
        return user.get('username')


class UsersService(BaseService):

    def on_create(self, docs):
        for user_doc in docs:
            user_doc.setdefault('display_name', get_display_name(user_doc))

    def on_created(self, docs):
        for user_doc in docs:
            self.update_user_defaults(user_doc)
            add_activity('created user {{user}}', user=user_doc.get('display_name', user_doc.get('username')))

    def on_updated(self, updates, user):
        self.handle_status_changed(updates, user)

    def handle_status_changed(self, updates, user):
        status = updates.get('status', None)
        if status and status != user.get('status', 'active'):
            if status == 'inactive':
                # remove active tokens
                get_resource_service('auth').delete_action({'username': user.get('username')})
            # send email notification
            send_email = get_resource_service('preferences').email_notification_is_enabled(user['_id'])
            if send_email:
                self._send_user_status_changed_email([user['email']], status)

    def _send_user_status_changed_email(self, recipients, status):
        send_email.delay(subject='Your Superdesk account is %s' % status,
                         sender=ADMINS[0],
                         recipients=recipients,
                         text_body=render_template("account_status_changed.txt", status=status),
                         html_body=render_template("account_status_changed.html", status=status))

    def on_deleted(self, doc):
        add_activity('removed user {{user}}', user=doc.get('display_name', doc.get('username')))

    def on_fetched(self, document):
        for doc in document['_items']:
            self.update_user_defaults(doc)

    def update_user_defaults(self, doc):
        """Set default fields for users"""
        doc.setdefault('display_name', get_display_name(doc))
        doc.pop('password', None)


class DBUsersService(UsersService):
    """
    Service class for UsersResource and should be used when AD is inactive.
    """

    def on_create(self, docs):
        super().on_create(docs)
        for doc in docs:
            if doc.get('password', None) and not is_hashed(doc.get('password')):
                doc['password'] = get_hash(doc.get('password'), app.config.get('BCRYPT_GENSALT_WORK_FACTOR', 12))


class ADUsersService(UsersService):
    """
    Service class for UsersResource and should be used when AD is active.
    """

    readonly_fields = ['username', 'display_name', 'password', 'email', 'phone', 'first_name', 'last_name']

    def on_fetched(self, doc):
        super().on_fetched(doc)
        for document in doc['_items']:
            document['_readonly'] = ADUsersService.readonly_fields




import string
import random
import bcrypt
from importlib import import_module


def get_random_string(length=12):
    chars = string.ascii_letters + string.digits
    return ''.join([random.choice(chars) for i in range(length)])


def import_by_path(path):
    module_path, class_name = path.rsplit('.', 1)
    module = import_module(module_path)
    return getattr(module, class_name)


def get_hash(input_str, salt):
    hashed = bcrypt.hashpw(input_str.encode('UTF-8'), bcrypt.gensalt(salt))
    return hashed.decode('UTF-8')


def is_hashed(input_str):
    """Check if given input_str is hashed."""
    return input_str.startswith('$2a$')


class ListCursor(object):
    """Wrapper for a python list as a cursor."""

    def __init__(self, docs=None):
        self.docs = docs if docs else []

    def __getitem__(self, key):
        return self.docs[key]

    def first(self):
        """Get first doc."""
        return self.docs[0] if self.docs else None

    def count(self, **kwargs):
        """Get total count."""
        return len(self.docs)

    def extra(self, response):
        pass



import os
import unittest
from app import get_app
from pyelasticsearch import ElasticSearch
from base64 import b64encode
from flask import json
from superdesk.notification_mock import setup_notification_mock, teardown_notification_mock
from superdesk import get_resource_service
from settings import LDAP_SERVER
from unittest.mock import patch
from apps.auth.ldap.ldap import ADAuth

test_user = {'username': 'test_user', 'password': 'test_password'}


def get_test_settings():
    test_settings = {}
    test_settings['ELASTICSEARCH_INDEX'] = 'sptests'
    test_settings['MONGO_DBNAME'] = 'sptests'
    test_settings['DEBUG'] = True
    test_settings['TESTING'] = True
    test_settings['BCRYPT_GENSALT_WORK_FACTOR'] = 4
    test_settings['CELERY_ALWAYS_EAGER'] = 'True'

    return test_settings


def drop_elastic(settings):
    try:
        es = ElasticSearch(settings['ELASTICSEARCH_URL'])
        es.delete_index(settings['ELASTICSEARCH_INDEX'])
    except:
        pass


def drop_mongo(app):
    with app.test_request_context():
        try:
            app.data.mongo.driver.cx.drop_database(app.config['MONGO_DBNAME'])
        except AttributeError:
            pass


def setup(context=None, config=None):
    if config:
        config.update(get_test_settings())
    else:
        config = get_test_settings()

    drop_elastic(config)
    app = get_app(config)

    drop_mongo(app)
    if context:
        context.app = app
        context.client = app.test_client()


def setup_auth_user(context, user=None):
    if LDAP_SERVER:
        setup_ad_user(context, user)
    else:
        setup_db_user(context, user)


def add_to_context(context, token, user):
    context.headers.append(('Authorization', b'basic ' + b64encode(token + b':')))
    context.user = user


def setup_db_user(context, user):
    user = user or test_user
    with context.app.test_request_context():
        original_password = user['password']
        user['user_type'] = 'administrator'
        get_resource_service('users').post([user])
        user['password'] = original_password

    auth_data = json.dumps({'username': user['username'], 'password': user['password']})
    auth_response = context.client.post('/auth', data=auth_data, headers=context.headers)
    token = json.loads(auth_response.get_data()).get('token').encode('ascii')

    add_to_context(context, token, user)


def setup_ad_user(context, user):
    ad_user = {'email': 'mock@mail.com.au', 'user_type': 'administrator'}
    if user and user['username']:
        ad_user['username'] = user['username']
    else:
        ad_user['username'] = test_user['username']

    with patch.object(ADAuth, 'authenticate_and_fetch_profile', return_value=ad_user):
        auth_data = json.dumps({'username': ad_user['username'], 'password': test_user['password']})
        auth_response = context.client.post('/auth', data=auth_data, headers=context.headers)
        auth_response_as_json = json.loads(auth_response.get_data())
        token = auth_response_as_json.get('token').encode('ascii')
        ad_user['_id'] = auth_response_as_json['user']

        add_to_context(context, token, ad_user)


def setup_notification(context):
    setup_notification_mock(context)


def teardown_notification(context):
    teardown_notification_mock(context)


class TestCase(unittest.TestCase):

    def setUp(self):
        setup(self)

    def get_fixture_path(self, filename):
        rootpath = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
        return os.path.join(rootpath, 'features', 'steps', 'fixtures', filename)



from builtins import NotImplementedError


class DataLayer():
    def etag(self, doc):
        raise NotImplementedError()

    def find_one(self, resource, filter, projection, options):
        raise NotImplementedError()

    def find(self, resource, filter, projection, options):
        raise NotImplementedError()

    def create(self, resource, docs):
        raise NotImplementedError()

    def update(self, resource, filter, doc):
        raise NotImplementedError()

    def replace(self, resource, filter, doc):
        raise NotImplementedError()

    def delete(self, resource, filter):
        raise NotImplementedError()




import arrow
import datetime
from pytz import utc, timezone # flake8: noqa

tzinfo = getattr(datetime, 'tzinfo', object)


def utcnow():
    """Get tz aware datetime object"""

    if hasattr(datetime.datetime, 'now'):
        return datetime.datetime.now(tz=utc)
    else:
        return datetime.datetime.utcnow()


def get_date(date_or_string):
    if date_or_string:
        return arrow.get(date_or_string).datetime




import logging
import flask

from eve.methods.post import post_internal
from superdesk.notification import push_notification
from superdesk.resource import Resource
from superdesk.services import BaseService
import superdesk

log = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = 'audit'
    service = BaseService(endpoint_name, backend=superdesk.get_backend())
    auditResource = AuditResource(endpoint_name, app=app, service=service)
    app.on_inserted += auditResource.on_generic_inserted
    app.on_updated += auditResource.on_generic_updated
    app.on_deleted_item += auditResource.on_generic_deleted

    endpoint_name = 'activity'
    service = BaseService(endpoint_name, backend=superdesk.get_backend())
    ActivityResource(endpoint_name, app=app, service=service)


class AuditResource(Resource):
    endpoint_name = 'audit'
    resource_methods = ['GET']
    item_methods = ['GET']
    schema = {
        'resource': {'type': 'string'},
        'action': {'type': 'string'},
        'extra': {'type': 'dict'},
        'user': Resource.rel('users', False)
    }
    exclude = {endpoint_name, 'activity'}

    def on_generic_inserted(self, resource, docs):
        if resource in self.exclude:
            return

        user = getattr(flask.g, 'user', None)
        if not user:
            return

        if not len(docs):
            return

        audit = {
            'user': user.get('_id'),
            'resource': resource,
            'action': 'created',
            'extra': docs[0]
        }

        post_internal(self.endpoint_name, audit)

    def on_generic_updated(self, resource, doc, original):
        if resource in self.exclude:
            return

        user = getattr(flask.g, 'user', None)
        if not user:
            return

        audit = {
            'user': user.get('_id'),
            'resource': resource,
            'action': 'updated',
            'extra': doc
        }
        post_internal(self.endpoint_name, audit)

    def on_generic_deleted(self, resource, doc):
        if resource in self.exclude:
            return

        user = getattr(flask.g, 'user', None)
        if not user:
            return

        audit = {
            'user': user.get('_id'),
            'resource': resource,
            'action': 'deleted',
            'extra': doc
        }
        post_internal(self.endpoint_name, audit)


class ActivityResource(Resource):
    endpoint_name = 'activity'
    resource_methods = ['GET']
    item_methods = ['GET', 'PATCH']
    schema = {
        'message': {'type': 'string'},
        'data': {'type': 'dict'},
        'read': {'type': 'dict'},
        'item': Resource.rel('archive', type='string'),
        'user': Resource.rel('users'),
    }
    exclude = {endpoint_name, 'notification'}
    datasource = {
        'default_sort': [('_created', -1)]
    }
    superdesk.register_available_preference('email:notification', {
        'type': 'bool',
        'enabled': True,
        'default': True,
        'label': 'Send notifications via email',
        'category': 'notifications'
    })


def add_activity(msg, item=None, notify=None, **data):
    """Add an activity into activity log.

    This will became part of current user activity log.

    If there is someone set to be notified it will make it into his notifications box.
    """
    activity = {
        'message': msg,
        'data': data,
    }

    user = getattr(flask.g, 'user', None)
    if user:
        activity['user'] = user.get('_id')

    if notify:
        activity['read'] = {str(_id): 0 for _id in notify}
    else:
        activity['read'] = {}

    if item:
        activity['item'] = str(item)

    post_internal(ActivityResource.endpoint_name, activity)
    push_notification(ActivityResource.endpoint_name, _dest=activity['read'])



class InvalidFilter(Exception):
    def __init__(self, filter, operation):
        self.filter = filter
        self.operation = operation

    def __str__(self):
        return 'Invalid filter on %s: %s' % (self.operation, filter)


class InvalidEtag(Exception):
    def __str__(self):
        return 'Client and server etags don\'t match'


class Validator():
    def validate(self, doc):
        raise NotImplementedError()


class ValidationError():
    def __init__(self, errors):
        self.errors = errors


class BaseModel():
    '''
    This is a basic interface for defining models. The only requirement is
    to implement the name method that uniquely identifies a model.
    '''

    def __init__(self, resource, data_layer, schema, validator):
        '''
        @param resource: str
            The resource name
        @param data_layer: object
            The object that implements the database operations.
        @param schema: dict
            The model schema definition
        @param validator: object
            The object that validates the model data.
        '''
        self.resource = resource
        self.data_layer = data_layer
        self.schema = schema
        self.validator = validator

    @classmethod
    def name(cls):
        raise NotImplementedError()

    def on_create(self, docs):
        '''
        Method called before the create operation.
        @param docs: list of docs to be inserted
        '''

    def on_created(self, docs):
        '''
        Method called after the create operation.
        @param docs: list of inserted docs
        '''

    def on_update(self, doc, orig):
        '''
        Method called before the update/replace operations.
        @param doc: the document updates
        @param orig: the original document before the update
        '''

    def on_updated(self, doc, orig):
        '''
        Method called after the update/replace operations.
        @param doc: the document updates
        @param orig: the original document before the update
        '''

    def on_delete(self, docs):
        '''
        Method called before the delete operation.
        @param docs: list of docs to be deleted
        '''

    def on_deleted(self, docs):
        '''
        Method called after the delete operation.
        @param docs: list of deleted docs
        '''

    def etag(self, doc):
        '''
        Return the document etag.
        '''
        return self.data_layer.etag(doc)

    def validate_etag(self, doc, etag):
        '''
        Validates the given etag for the given document.
        Raises InvalidEtag if they don't match.
        '''
        if self.etag(doc) != etag:
            raise InvalidEtag()

    def find_one(self, filter, projection=None):
        '''
        Return one document selected based on the given filter.
        @param filter: dict
        @param projection: dict
        '''
        return self.data_layer.find_one(self.resource, filter, projection)

    def find(self, filter, projection=None, **options):
        '''
        Return a list of documents selected based on the given filter.
        @param filter: dict
        @param projection: dict
        @param options: dict
        '''
        return self.data_layer.find(self.resource, filter, projection, **options)

    def create(self, docs):
        '''
        Insert a list of documents
        @param docs: list
        '''
        self.validate(docs)
        self.on_create(docs)
        res = self.data_layer.create(self.resource, docs)
        self.on_created(docs)
        return res

    def update(self, filter, doc):
        '''
        Update one document selected based on the given filter.
        @param filter: dict
        @param doc: dict
        '''
        self.validate(doc)
        orig = self.find(filter)
        if orig.count() > 1:
            raise InvalidFilter(filter, 'update')
        if orig.count() == 0:
            return
        self.on_update(doc, orig[0])
        res = self.data_layer.update(self.resource, filter, doc)
        self.on_updated(doc, orig[0])
        return res

    def replace(self, filter, doc):
        '''
        Replace the content of one document selected based on the given filter.
        @param filter: dict
        @param doc: dict
        '''
        self.validate(doc)
        orig = self.find(filter)
        if len(orig) != 1:
            raise InvalidFilter(filter, 'replace')
        self.on_update(doc, orig[0])
        res = self.data_layer.replace(self.resource, filter, doc)
        self.on_updated(doc, orig[0])
        return res

    def delete(self, filter):
        '''
        Delete one document selected based on the given filter.
        @param filter: dict
        '''
        orig = self.find(filter)
        if not orig:
            return
        self.on_delete(orig)
        res = self.data_layer.delete(self.resource, filter)
        self.on_deleted(orig)
        return res

    def validate(self, docs):
        '''
        Validate a document or a list of documents.
        Raise ValidationError on errors
        @param docs: list of documents
        '''
        errs = []
        if not isinstance(docs, list):
            docs = [docs]
        for doc in docs:
            err = self.validator.validate(doc)
            if errs:
                errs.append(err)
        if errs:
            raise ValidationError(errs)



from __future__ import absolute_import
from PIL import Image
from io import BytesIO
import logging
from flask import current_app as app
from .media_operations import process_file_from_stream

logger = logging.getLogger(__name__)


def generate_renditions(original, media_id, inserted, file_type, content_type, rendition_config, url_for_media):
    """Generate system renditions for given media file id."""
    rend = {'href': url_for_media(media_id), 'media': media_id, 'mimetype': content_type}
    renditions = {'original': rend}

    if file_type != 'image':
        return renditions

    original.seek(0)
    img = Image.open(original)
    width, height = img.size
    rend.update({'width': width})
    rend.update({'height': height})

    ext = content_type.split('/')[1].lower()
    if ext in ('JPG', 'jpg'):
        ext = 'jpeg'
    ext = ext if ext in ('jpeg', 'gif', 'tiff', 'png') else 'png'
    for rendition, rsize in rendition_config.items():
        size = (rsize['width'], rsize['height'])
        original.seek(0)
        resized, width, height = resize_image(original, ext, size)
        rend_content_type = 'image/%s' % ext
        file_name, rend_content_type, metadata = process_file_from_stream(resized, content_type=rend_content_type)
        resized.seek(0)
        id = app.media.put(resized, filename=file_name, content_type=rend_content_type, metadata=metadata)
        inserted.append(id)
        renditions[rendition] = {'href': url_for_media(id), 'media': id,
                                 'mimetype': 'image/%s' % ext, 'width': width, 'height': height}
    return renditions


def delete_file_on_error(doc, file_id):
    # Don't delete the file if we are on the import from storage flow
    if doc.get('_import', None):
        return
    app.media.delete(file_id)


def resize_image(content, format, size, keepProportions=True):
    '''
    Resize the image given as a binary stream

    @param content: stream
        The binary stream containing the image
    @param format: str
        The format of the resized image (e.g. png, jpg etc.)
    @param size: tuple
        A tuple of width, height
    @param keepProportions: boolean
        If true keep image proportions; it will adjust the resized
        image size.
    @return: stream
        Returns the resized image as a binary stream.
    '''
    assert isinstance(size, tuple)
    img = Image.open(content)
    if keepProportions:
        width, height = img.size
        new_width, new_height = size
        x_ratio = width / new_width
        y_ratio = height / new_height
        if x_ratio > y_ratio:
            new_height = int(height / x_ratio)
        else:
            new_width = int(width / y_ratio)
        size = (new_width, new_height)

    resized = img.resize(size)
    out = BytesIO()
    resized.save(out, format)
    out.seek(0)
    return out, new_width, new_height



import logging
from ldap3 import Server, Connection, SEARCH_SCOPE_WHOLE_SUBTREE, LDAPException
from apps.auth.errors import AuthError, NotFoundAuthError
from apps.auth.service import AuthService
from superdesk.utc import utcnow
from flask import current_app as app
import superdesk

logger = logging.getLogger(__name__)


class ADAuth:
    """
    Handles Authentication against Active Directory.
    """

    def __init__(self, host, port, base_filter, user_filter, profile_attributes, fqdn):
        """
        Initializes the AD Server
        :param host: ldap server. for example ldap://aap.com.au
        :param port: default port is 389
        :param base_filter:
        :param user_filter:
        :param profile_attributes:
        """
        self.ldap_server = Server(host, (port if port is not None else 389))

        self.fqdn = fqdn
        self.base_filter = base_filter
        self.user_filter = user_filter
        self.profile_attrs = profile_attributes

    def authenticate_and_fetch_profile(self, username, password, username_for_profile=None):
        """
        Authenticates a user with credentials username and password against AD. If authentication is successful then it
        fetches a profile of a user identified by username_for_profile and if found the profile is returned.
        :param username: LDAP username
        :param password: LDAP password
        :param username_for_profile: Username of the profile to be fetched
        :return: user profile base on the LDAP_USER_ATTRIBUTES
        """

        if username_for_profile is None:
            username_for_profile = username

        if self.fqdn is not None:
            username = username + "@" + self.fqdn

        try:
            ldap_conn = Connection(self.ldap_server, auto_bind=True, user=username, password=password)

            user_filter = self.user_filter.format(username_for_profile)
            logger.info('base filter:{} user filter:{}'.format(self.base_filter, user_filter))

            with ldap_conn:
                result = ldap_conn.search(self.base_filter, user_filter, SEARCH_SCOPE_WHOLE_SUBTREE,
                                          attributes=list(self.profile_attrs.keys()))

                response = dict()

                if result:
                    user_profile = ldap_conn.response[0]['attributes']

                    for ad_profile_attr, sd_profile_attr in self.profile_attrs.items():
                        response[sd_profile_attr] = \
                            user_profile[ad_profile_attr] if user_profile.__contains__(ad_profile_attr) else ''

                        response[sd_profile_attr] = response[sd_profile_attr][0] \
                            if isinstance(response[sd_profile_attr], list) else response[sd_profile_attr]

                return response
        except LDAPException as e:
            logger.error("Exception occurred. Login failed for user {}".format(username), e)
            raise AuthError()


class LdapAuthService(AuthService):

    def authenticate(self, credentials):
        """
        Authenticates the user against Active Directory
        :param credentials: an object having "username" and "password" attributes
        :return: if success returns User object, otherwise throws Error
        """
        settings = app.settings
        ad_auth = ADAuth(settings['LDAP_SERVER'], settings['LDAP_SERVER_PORT'], settings['LDAP_BASE_FILTER'],
                         settings['LDAP_USER_FILTER'], settings['LDAP_USER_ATTRIBUTES'], settings['LDAP_FQDN'])

        username = credentials.get('username')
        password = credentials.get('password')

        user_data = ad_auth.authenticate_and_fetch_profile(username, password)
        if len(user_data) == 0:
            raise NotFoundAuthError()

        user = superdesk.get_resource_service('users').find_one(username=username, req=None)

        if not user:
            user_data['username'] = username
            user_data[app.config['DATE_CREATED']] = user_data[app.config['LAST_UPDATED']] = utcnow()

            superdesk.get_resource_service('users').post([user_data])
        else:
            user_data[app.config['LAST_UPDATED']] = utcnow()
            superdesk.get_resource_service('users').patch(user.get('_id'), user_data)

        user = superdesk.get_resource_service('users').find_one(username=username, req=None)
        return user



import superdesk
from apps.auth import AuthResource
from .ldap import LdapAuthService, ADAuth  # noqa
from .commands import ImportUserProfileFromADCommand  # noqa


def init_app(app):
    endpoint_name = 'auth'
    service = LdapAuthService(endpoint_name, backend=superdesk.get_backend())
    AuthResource(endpoint_name, app=app, service=service)




from flask import Blueprint
from eve.render import send_response
from datetime import datetime
import superdesk


bp = Blueprint('subjectcodes', __name__)


@bp.route('/subjectcodes/', methods=['GET', 'OPTIONS'])
def get_subjectcodes():
    from .iptc import subject_codes

    items = []
    for code in sorted(subject_codes):
        items.append({'code': code, 'name': subject_codes[code]})

    return send_response(None, ({'_items': items}, datetime(2012, 7, 10), None, 200))

superdesk.blueprint(bp)




import re
from bson import ObjectId
from eve.io.mongo import Validator
from eve.utils import config
from werkzeug.datastructures import FileStorage
import superdesk


ERROR_PATTERN = {'pattern': 1}
ERROR_UNIQUE = {'unique': 1}
ERROR_MINLENGTH = {'minlength': 1}
ERROR_REQUIRED = {'required': 1}


class SuperdeskValidator(Validator):
    def _validate_type_phone_number(self, field, value):
        """ Enables validation for `phone_number` schema attribute.
            :param field: field name.
            :param value: field value.
        """
        if not re.match("^(?:(?:0?[1-9][0-9]{8})|(?:(?:\+|00)[1-9][0-9]{9,11}))$", value):
            self._error(field, ERROR_PATTERN)

    def _validate_type_email(self, field, value):
        """ Enables validation for `email` schema attribute.
            :param field: field name.
            :param value: field value.
        """
        regex = "^[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@" \
                "(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]{0,4}[a-z0-9]){1}$"
        if not re.match(regex, value):
            self._error(field, ERROR_PATTERN)

    def _validate_type_file(self, field, value):
        """Enables validation for `file` schema attribute."""
        if not isinstance(value, FileStorage):
            self._error(field, ERROR_PATTERN)

    def _validate_unique(self, unique, field, value):
        """Validate unique with custom error msg."""

        if unique:
            query = {field: value}
            if self._id:
                try:
                    query[config.ID_FIELD] = {'$ne': ObjectId(self._id)}
                except:
                    query[config.ID_FIELD] = {'$ne': self._id}

            if superdesk.get_resource_service(self.resource).find_one(req=None, **query):
                self._error(field, ERROR_UNIQUE)

    def _validate_minlength(self, min_length, field, value):
        """Validate minlength with custom error msg."""
        if isinstance(value, (type(''), list)):
            if len(value) < min_length:
                self._error(field, ERROR_MINLENGTH)

    def _validate_required_fields(self, document):
        required = list(field for field, definition in self.schema.items()
                        if definition.get('required') is True)
        missing = set(required) - set(key for key in document.keys()
                                      if document.get(key) is not None
                                      or not self.ignore_none_values)
        for field in missing:
            self._error(field, ERROR_REQUIRED)



import logging
from flask import json
import superdesk

logger = logging.getLogger(__name__)


class AuthRequiredError(superdesk.SuperdeskError):
    """Auth required error."""
    status_code = 401
    payload = {'auth': 1}


class AuthError(superdesk.SuperdeskError):
    """Base Auth Error"""
    status_code = 400
    payload = {'credentials': 1}


class NotFoundAuthError(AuthError):
    """Username Not Found Auth Error"""
    pass


class CredentialsAuthError(AuthError):
    """Credentials Not Match Auth Exception"""
    pass


class ForbiddenError(superdesk.SuperdeskError):
    status_code = 403


class UserInactiveError(ForbiddenError):
    """User is inactive, access restricted"""
    payload = {'status': 'inactive'}
    message = 'Account suspended, access restricted.'


def raiseCredentialsAuthError(credentials):
    logger.warning("Login failure: %s" % json.dumps(credentials))
    raise CredentialsAuthError()



'''Import from Amazon S3 into Content Archive'''

import superdesk
import json
from superdesk.media.image import get_meta
from eve.utils import ParsedRequest
import xml.etree.ElementTree as etree
from flask.globals import current_app as app
from superdesk.celery_app import celery
from celery import group


class ImportFromAmazonCommand(superdesk.Command):
    """
    Import content from Amazon S3 into content archive.
    """

    max_bucket_size = 50

    option_list = (
        superdesk.Option('--bucket-size', '-s', dest='bucket_size'),
    )

    def run(self, bucket_size=None):
        bucket_size = int(bucket_size) or self.max_bucket_size
        marker = process_bucket(bucket_size=bucket_size)
        while marker:
            marker = process_bucket(bucket_size, marker)
        print('Import completed')


def process_bucket(bucket_size, marker=None):
    print('Retrieving bucket with marker: ', marker)
    rv = app.media.get_bucket_objects(marker=marker)
    keys, marker = extract_keys(rv)
    if not keys:
        return marker
    paged_keys = [keys[i:i + bucket_size] for i in range(0, len(keys), bucket_size)]
    for key_group in paged_keys:
        res = group(import_file.si(key=key) for key in key_group)()
        rv = res.get()
        for value in rv:
            print(value['status'])
    return marker


@celery.task
def import_file(key):
    try:
        if check_if_file_already_imported(key):
            msg = 'File %s already imported' % key
            return {'status': msg}

        file = retrieve_file(key)
        if not file:
            return {'status': 'Failed to retrieve file: ' + key}

        data = [{'media': key, 'media_fetched': file, '_import': True}]
        id = superdesk.get_resource_service('archive_media').post(data)
        return {'status': 'Imported file %s to archive media with id= %s' % (key, id)}
    except Exception as ex:
        return {'status': ex}


def retrieve_file(key):
    file = app.media.get(key)
    if file and not file.metadata:
        metadata = get_meta(file)
        if metadata:
            metadata['content-type'] = file.content_type
            file.metadata = metadata
            app.media.update_metadata(key, metadata)
        file.seek(0)
    return file


def extract_keys(xml_content):
    rv = etree.fromstring(xml_content)
    namespace = ''
    if rv.tag[0] == '{':
        uri, tag = rv.tag[0:].split('}')
        namespace = uri + '}'
    keys = [key.text for key in rv.iter(namespace + 'Key')]
    truncated = rv.find(namespace + 'IsTruncated')
    marker = None
    if keys and json.loads(truncated.text):
        marker = keys[-1]
    return keys, marker


def check_if_file_already_imported(key):
    query_filter = get_query_for_already_imported(key)
    req = ParsedRequest()
    req.args = {'filter': query_filter}
    res = superdesk.get_resource_service('archive').get(req, None).count()
    return res > 0


def get_query_for_already_imported(file_id):
    query = {'bool':
             {'should':
              [{'term': {'archive.media': file_id}},
               {'term': {'archive.renditions.baseImage.media': file_id}},
               {'term': {'archive.renditions.original.media': file_id}},
               {'term': {'archive.renditions.thumbnail.media': file_id}},
               {'term': {'archive.renditions.viewImage.media': file_id}}]}
             }
    return json.dumps(query)
